/*
 * BacteryApplet.java
 *
 * Created on 12 novembre 2008, 13:58
 * 
 * Copyright 2015 Riccardo Scorretti, Olivier Fabregues, Sandrine Demaneche, Elisabeth Navarro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */

package risaaligner;

import ampere.util.icon.Icone;
import ampere.util.io.ErrorMsg;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.prefs.Preferences;
import javax.swing.Box.Filler;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import risaaligner.datas.Sample;


/**
 *
 * @author  olivier
 */
public class Applet extends javax.swing.JApplet  implements MainInterface,ActionListener {
    static final long serialVersionUID = 00L;
    
    public static String[] parametersNames = {"HEIGHT", "WIDTH"};
    
    RisaAligner risaAligner;
    
    JButton alignLadderButton;
    JToggleButton lockButton;
    JButton alignDatasButton;

    JToggleButton shadeButton;
    JToggleButton view0Button;
    JToggleButton view1Button;
    JToggleButton view2Button;
    Filler filler0;
    Filler filler1;
    Filler filler2;
    Filler filler3;
    Filler filler4;
    JButton exportButton;

    boolean licenced;
    boolean locked;
    int state;
    Sample sample;

    boolean local = false;
    boolean debug = false;
    File userHome;
    
    String startFileName = null;
    
    public void init() {
        //Execute a job on the event-dispatching thread:
        //creating this applet's GUI.
        try {
            javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
                public void run() {

                    
                    initComponents();
                    
                    /*
                     * construction des boutons de la barre de menus
                     */
                    buildButtons();
                    
                    /*
                     * icone Ampere dans le coin gauche de la fenetre
                     */
                    ImageIcon ampere = new Icone("ampere3").getIcon();
                    //setIconImage(ampere.getImage());
                    
                    /*
                     * lancement du programme
                     */
                    startProgram();
                    

                    //createContext();

                }
            });
        } catch (Exception e) {
            ErrorMsg.write ("applet didn't successfully complete"+e);
        }
    }
    public void stop() {
        risaAligner.quit();
    }
    
    void startProgram() {
        
        risaAligner = new RisaAligner(this);

        //getContentPane().add(risaAligner,BorderLayout.CENTER);
        getContentPane().add(risaAligner);
        
        /*
         * chargement de la config
         * 
         */
        loadConfig();
        
        pack();

    }

    public void pack() {
        Container cp = getContentPane();
        Dimension d1 = cp.getSize();
        //risaAligner.writeMsg("windows size : "+d1);
        risaAligner.setSize(d1);
        
        //Dimension d2 = cp.getLayout().preferredLayoutSize(cp);
        //risaAligner.writeMsg("windows content size : "+d2);
        //risaAligner.validate();
    } 

    void buildButtons() {
        

        filler0 = new Filler(new java.awt.Dimension(20, 0), new java.awt.Dimension(20, 0), new java.awt.Dimension(20,32767));
        jMenuBar1.add(filler0);

        JPanel jPanelA = new JPanel();
        jPanelA.setLayout(new javax.swing.BoxLayout(jPanelA, javax.swing.BoxLayout.LINE_AXIS));
        jPanelA.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        lockButton = new JToggleButton();
        ImageIcon lock = new Icone("cadenas").getIcon();
        ImageIcon unlock = new Icone("cadenas2").getIcon();
        lockButton.setIcon(unlock);
        lockButton.setSelectedIcon(lock);
        lockButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        lockButton.setEnabled(false);
        lockButton.addActionListener(this);
        lockButton.setToolTipText("lock reference");
        jPanelA.add(lockButton);

        alignLadderButton = new JButton();
        ImageIcon align = new Icone("align_ladders").getIcon();
        alignLadderButton.setIcon(align);
        alignLadderButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        alignLadderButton.setEnabled(false);
        alignLadderButton.addActionListener(this);
        alignLadderButton.setToolTipText("align ladders");
        jPanelA.add(alignLadderButton);

        jMenuBar1.add(jPanelA);
        
        filler1 = new Filler(new java.awt.Dimension(20, 0), new java.awt.Dimension(20, 0), new java.awt.Dimension(20,32767));
        jMenuBar1.add(filler1);
        
        JPanel jPanelB = new JPanel();
        jPanelB.setLayout(new javax.swing.BoxLayout(jPanelB, javax.swing.BoxLayout.LINE_AXIS));
        jPanelB.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        /*
         * construction des boutons
         */
        shadeButton = new JToggleButton();
        ImageIcon shadeVisible = new Icone("shade").getIcon();
        shadeButton.setIcon(shadeVisible);
        shadeButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        shadeButton.setEnabled(false);
        shadeButton.addActionListener(this);
        shadeButton.setToolTipText("Show shade");

        jPanelB.add(shadeButton);

        alignDatasButton = new JButton();
        ImageIcon compute = new Icone("align_datas").getIcon();
        alignDatasButton.setIcon(compute);
        alignDatasButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        alignDatasButton.setEnabled(false);
        alignDatasButton.addActionListener(this);
        alignDatasButton.setToolTipText("align datas");
        jPanelB.add(alignDatasButton);

        jMenuBar1.add(jPanelB);

        filler2 = new Filler(new java.awt.Dimension(20, 0), new java.awt.Dimension(20, 0), new java.awt.Dimension(20,32767));
        jMenuBar1.add(filler2);

        JPanel jPanelC = new JPanel();
        jPanelC.setLayout(new javax.swing.BoxLayout(jPanelC, javax.swing.BoxLayout.LINE_AXIS));
        jPanelC.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        ButtonGroup buttonGroup = new ButtonGroup();
        view0Button = new JToggleButton();
        ImageIcon viewMode0 = new Icone("wiew_noAligned").getIcon();
        view0Button.setIcon(viewMode0);
        view0Button.setMargin(new java.awt.Insets(0, 0, 0, 0));
        view0Button.setEnabled(false);
        view0Button.addActionListener(this);
        view0Button.setToolTipText("View no aligned datas");
        buttonGroup.add(view0Button);

        jPanelC.add(view0Button);
        
        view1Button = new JToggleButton();
        ImageIcon viewMode = new Icone("wiew_aligned").getIcon();
        view1Button.setIcon(viewMode);
        view1Button.setMargin(new java.awt.Insets(0, 0, 0, 0));
        view1Button.setEnabled(false);
        view1Button.addActionListener(this);
        view1Button.setToolTipText("View aligned datas");
        buttonGroup.add(view1Button);

        jPanelC.add(view1Button);

        view2Button = new JToggleButton();
        ImageIcon viewMode2 = new Icone("wiew_aligned2").getIcon();
        view2Button.setIcon(viewMode2);
        view2Button.setMargin(new java.awt.Insets(0, 0, 0, 0));
        view2Button.setEnabled(false);
        view2Button.addActionListener(this);
        view2Button.setToolTipText("View twice aligned datas");
        buttonGroup.add(view2Button);

        jPanelC.add(view2Button);
        
        jMenuBar1.add(jPanelC);

        filler3 = new Filler(new java.awt.Dimension(20, 0), new java.awt.Dimension(20, 0), new java.awt.Dimension(20,32767));
        jMenuBar1.add(filler3);

        JPanel jPanelD = new JPanel();
        jPanelD.setLayout(new javax.swing.BoxLayout(jPanelD, javax.swing.BoxLayout.LINE_AXIS));
        jPanelD.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        exportButton = new JButton();
        ImageIcon save = new Icone("export").getIcon();
        exportButton.setIcon(save);
        exportButton.setMargin(new java.awt.Insets(0, 0, 0, 0));
        exportButton.setEnabled(false);
        exportButton.addActionListener(this);
        exportButton.setToolTipText("save aligned datas");
        jPanelD.add(exportButton);

        jMenuBar1.add(jPanelD);

        filler4 = new Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        jMenuBar1.add(filler4);
    }
    private void loadConfig() {
        /*
         * lecture de la config enregistree ds la base de registre
         */
        Preferences userPrefs = Preferences.userNodeForPackage(getClass());
        if (userPrefs != null) {
            risaAligner.writeMsg("> Load user config ..."+'\n');

            String projectFileName = null;
            projectFileName = userPrefs.get("projectFile", projectFileName);
            risaAligner.loadConfig(projectFileName);
        }
        /*
         * lecture des parametres passes à l applet
         */
        for (int i=0;i<parametersNames.length;i++) {
            String parameterName = parametersNames[i];
            String parameterValue = getParameter (parameterName);
            if (parameterValue != null) {
                if (parameterName.equals("WIDTH")) {
                }
                if (parameterName.equals("HEIGHT")) {
                }
            }
        }
    }
    public void saveConfig(String projectFilePath)  {
        if (projectFilePath != null) {
            risaAligner.writeMsg("> Save default directory :  "+projectFilePath+'\n');
            Preferences userPrefs = Preferences.userNodeForPackage(getClass());
            userPrefs.put("projectFile", projectFilePath);
        }
    }
    /*
     * lecture des parametres passes
     *
    private void loadConfig() {
        
            
        // recherche du repertoire utilisateur
        userHome = new File(System.getProperty("user.home"));
        if (debug) Output.write("user home "+userHome);

        // Recuperation de l'URI du repertoire de base
        //
        // Dans le cas de l'applet viewer ou il n y a pas de veritable serveur
        // L'adresse URL du document de base qui est recuperee est opaque
        // Elle ne peut pas etre utilisee pour faire des resolve sur l'URI
        //
        // Dans ce cas on utilise l'url du serveur Apache.
        // Il faut donc qu'il soit demarre !

        try {
            URL url = getDocumentBase();
            URI uriBase = url.toURI();
            if (uriBase.isOpaque()) {
                uriBase = new URL("http://127.0.0.1/").toURI();
            }
            if (debug) Output.write("URI de base "+uriBase);
        }
        catch (URISyntaxException ex) {
            ErrorMsg.write ("Problem building base URL "+ex);
        }
        catch (MalformedURLException ex) {
            ErrorMsg.write ("Problem building base URL "+ex);
        }

        // lecture eventuelle du fichier de config
        //loadConfigFile();

        // recuperation des noms de parametres existants
        //String[] parameterNames = getParameterNames();

        // Lecture des parametres passes a l'applet dans la ligne HTML
        // ils sont prioritaires par rapport au fichier de config
        if (debug) Output.write("set HTML parameters : ");

        /*
        for (int i=0;i<parameterNames.length;i++) {
            String parameterName = parameterNames[i];
            String parameterValue = getParameter (parameterName);
            if (parameterValue != null) setParameter (parameterName,parameterValue);
        }
    }
     */

    public boolean verifyLicence() {
        boolean licenceOk = true;
        return licenceOk;
    }
        
    public void reinitLicence() {
    }
    public void setLicence() {
        boolean licenceOk = true;
        updateButtons(licenceOk, locked,state, null);
    }
    public void showLicence() {
        risaAligner.writeMsg("> Expiration date : Never" + '\n');
    }
    public void updateButtons (boolean licenced) {
        this.licenced = licenced;
        updateButtons(licenced,locked,state,sample);
    }
    public void updateButtons (boolean locked,int state, Sample sample) {
        this.locked = locked;
        this.state = state;
        this.sample = sample;
    }
    public void updateButtons (boolean licenced,boolean locked,int state, Sample sample) {
        this.licenced = licenced;
        this.locked = locked;
        this.state = state;
        this.sample = sample;

        loadProjectMenuItem.setEnabled(false);
        saveProjectMenuItem.setEnabled(false);
        openDirectoriesMenuItem.setEnabled(false);
        closeDirectoriesMenuItem.setEnabled(false);
        openDirectoryMenuItem.setEnabled(false);
        closeDirectoryMenuItem.setEnabled(false);
        exportMenuItem.setEnabled(false);
        printMenu.setEnabled(false);
        alignLadderMenuItem.setEnabled(false);
        alignDatasMenuItem.setEnabled(false);
        
        initLicenceMenuItem.setEnabled(true);
        setLicenceMenuItem.setEnabled(true);
        showLicenceMenuItem.setEnabled(true);
        
        alignLadderButton.setEnabled(false);
        alignDatasButton.setEnabled(false);
        
        lockButton.setEnabled(false);
        shadeButton.setEnabled(false);
        exportButton.setEnabled(false);

        if (licenced) {
            if (state == RisaAligner.STATE_NOTHING) {
                loadProjectMenuItem.setEnabled(true);
                openDirectoriesMenuItem.setEnabled(true);
                openDirectoryMenuItem.setEnabled(true);
                //closeDirectoriesMenuItem.setEnabled(true);
                //closeDirectoryMenuItem.setEnabled(true);
                lockButton.setSelected(false);
            }
            else if (state == RisaAligner.STATE_SAMPLE) {
                openDirectoriesMenuItem.setEnabled(true);
                openDirectoryMenuItem.setEnabled(true);
                closeDirectoriesMenuItem.setEnabled(true);
                closeDirectoryMenuItem.setEnabled(true);
                saveProjectMenuItem.setEnabled(true);
                printMenu.setEnabled(true);
                shadeButton.setEnabled(true);
            }
            else if (state == RisaAligner.STATE_ALIGNABLE) {
                openDirectoriesMenuItem.setEnabled(true);
                openDirectoryMenuItem.setEnabled(true);
                closeDirectoriesMenuItem.setEnabled(true);
                closeDirectoryMenuItem.setEnabled(true);
                saveProjectMenuItem.setEnabled(true);
                printMenu.setEnabled(true);
                lockButton.setEnabled(true);
                shadeButton.setEnabled(true);
                alignLadderButton.setEnabled(true);
                alignLadderMenuItem.setEnabled(true);
            }
            else if (state == RisaAligner.STATE_LADDER_ALIGNED) {
                openDirectoriesMenuItem.setEnabled(true);
                openDirectoryMenuItem.setEnabled(true);
                closeDirectoriesMenuItem.setEnabled(true);
                closeDirectoryMenuItem.setEnabled(true);
                saveProjectMenuItem.setEnabled(true);
                printMenu.setEnabled(true);
                lockButton.setEnabled(true);
                shadeButton.setEnabled(true);
                if (locked){
                    alignLadderButton.setEnabled(false);
                    alignLadderMenuItem.setEnabled(false);
                }
                else {
                    alignLadderButton.setEnabled(true);
                    alignLadderMenuItem.setEnabled(true);
                }
                exportButton.setEnabled(true);
                exportMenuItem.setEnabled(true);
            }

            if (locked) lockButton.setSelected(true);
            
            if (sample == null) { 
                view0Button.setSelected(true);
                view0Button.setEnabled(false);
                view1Button.setEnabled(false);
                view2Button.setEnabled(false);
                alignDatasButton.setEnabled(false);
                alignDatasMenuItem.setEnabled(false);
            }
            else {
                if (sample.bestViewMode == RisaAligner.VIEW_SAMPLE){
                    view0Button.setSelected(true);
                }
                else if (sample.bestViewMode == RisaAligner.VIEW_LADDER_ALIGNED){
                    view1Button.setSelected(true);
                }
                else {
                    view2Button.setSelected(true);
                }

                if (sample.isDatasAligned()) {
                    alignDatasButton.setEnabled(false);
                    alignDatasMenuItem.setEnabled(false);
                    view0Button.setEnabled(true);
                    view1Button.setEnabled(true);
                    view2Button.setEnabled(true);
                }
                else if (sample.isLadderAligned()) {
                    alignDatasButton.setEnabled(true);
                    alignDatasMenuItem.setEnabled(true);
                    view0Button.setEnabled(true);
                    view1Button.setEnabled(true);
                    view2Button.setEnabled(false);
                }
                else {
                    alignDatasButton.setEnabled(false);
                    alignDatasMenuItem.setEnabled(false);
                    view0Button.setEnabled(true);
                    view1Button.setEnabled(false);
                    view2Button.setEnabled(false);
                }
            }
        }
    }

    /** This method is called from within the init() method to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        loadProjectMenuItem = new javax.swing.JMenuItem();
        saveProjectMenuItem = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JPopupMenu.Separator();
        openDirectoriesMenuItem = new javax.swing.JMenuItem();
        closeDirectoriesMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JPopupMenu.Separator();
        openDirectoryMenuItem = new javax.swing.JMenuItem();
        closeDirectoryMenuItem = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JPopupMenu.Separator();
        exportMenuItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JPopupMenu.Separator();
        printMenu = new javax.swing.JMenu();
        printLadderMenuItem = new javax.swing.JMenuItem();
        printSampleMenuItem = new javax.swing.JMenuItem();
        jMenu2 = new javax.swing.JMenu();
        alignLadderMenuItem = new javax.swing.JMenuItem();
        alignDatasMenuItem = new javax.swing.JMenuItem();
        jMenu3 = new javax.swing.JMenu();
        initLicenceMenuItem = new javax.swing.JMenuItem();
        setLicenceMenuItem = new javax.swing.JMenuItem();
        showLicenceMenuItem = new javax.swing.JMenuItem();

        addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                formPropertyChange(evt);
            }
        });

        jMenu1.setText("File");

        loadProjectMenuItem.setText("load project");
        loadProjectMenuItem.setEnabled(false);
        loadProjectMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadProjectMenuItemActionPerformed(evt);
            }
        });
        jMenu1.add(loadProjectMenuItem);

        saveProjectMenuItem.setText("save project");
        saveProjectMenuItem.setEnabled(false);
        saveProjectMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveProjectMenuItemActionPerformed(evt);
            }
        });
        jMenu1.add(saveProjectMenuItem);
        jMenu1.add(jSeparator5);

        openDirectoriesMenuItem.setText("open several arrays");
        openDirectoriesMenuItem.setEnabled(false);
        openDirectoriesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openDirectoriesMenuItemActionPerformed(evt);
            }
        });
        jMenu1.add(openDirectoriesMenuItem);

        closeDirectoriesMenuItem.setText("close all arrays");
        closeDirectoriesMenuItem.setEnabled(false);
        closeDirectoriesMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeDirectoriesMenuItemActionPerformed(evt);
            }
        });
        jMenu1.add(closeDirectoriesMenuItem);
        jMenu1.add(jSeparator1);

        openDirectoryMenuItem.setText("open array");
        openDirectoryMenuItem.setActionCommand("open array");
        openDirectoryMenuItem.setEnabled(false);
        openDirectoryMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                openDirectoryMenuItemActionPerformed(evt);
            }
        });
        jMenu1.add(openDirectoryMenuItem);

        closeDirectoryMenuItem.setText("close array");
        closeDirectoryMenuItem.setActionCommand("close array");
        closeDirectoryMenuItem.setEnabled(false);
        closeDirectoryMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeDirectoryMenuItemActionPerformed(evt);
            }
        });
        jMenu1.add(closeDirectoryMenuItem);
        jMenu1.add(jSeparator4);

        exportMenuItem.setText("Export");
        exportMenuItem.setEnabled(false);
        exportMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exportMenuItemActionPerformed(evt);
            }
        });
        jMenu1.add(exportMenuItem);
        jMenu1.add(jSeparator2);

        printMenu.setText("print");
        printMenu.setEnabled(false);

        printLadderMenuItem.setText("ladder");
        printLadderMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                printLadderMenuItemActionPerformed(evt);
            }
        });
        printMenu.add(printLadderMenuItem);

        printSampleMenuItem.setText("sample");
        printSampleMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                printSampleMenuItemActionPerformed(evt);
            }
        });
        printMenu.add(printSampleMenuItem);

        jMenu1.add(printMenu);

        jMenuBar1.add(jMenu1);

        jMenu2.setText("Edit");

        alignLadderMenuItem.setText("Align ladder");
        alignLadderMenuItem.setEnabled(false);
        alignLadderMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alignLadderMenuItemActionPerformed(evt);
            }
        });
        jMenu2.add(alignLadderMenuItem);

        alignDatasMenuItem.setText("align sample");
        alignDatasMenuItem.setEnabled(false);
        alignDatasMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alignDatasMenuItemActionPerformed(evt);
            }
        });
        jMenu2.add(alignDatasMenuItem);

        jMenuBar1.add(jMenu2);

        jMenu3.setText("Tools");

        initLicenceMenuItem.setText("init licence");
        initLicenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                initLicenceMenuItemActionPerformed(evt);
            }
        });
        jMenu3.add(initLicenceMenuItem);

        setLicenceMenuItem.setText("enter licence");
        setLicenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                setLicenceMenuItemActionPerformed(evt);
            }
        });
        jMenu3.add(setLicenceMenuItem);

        showLicenceMenuItem.setText("expiration");
        showLicenceMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showLicenceMenuItemActionPerformed(evt);
            }
        });
        jMenu3.add(showLicenceMenuItem);

        jMenuBar1.add(jMenu3);

        setJMenuBar(jMenuBar1);
    }// </editor-fold>//GEN-END:initComponents

    private void openDirectoryMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openDirectoryMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.chooseDirectory();
    }//GEN-LAST:event_openDirectoryMenuItemActionPerformed

    private void closeDirectoryMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeDirectoryMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.closeCurrentDirectory();
    }//GEN-LAST:event_closeDirectoryMenuItemActionPerformed

    private void alignLadderMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alignLadderMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.alignLadders();
    }//GEN-LAST:event_alignLadderMenuItemActionPerformed

    private void openDirectoriesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openDirectoriesMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.chooseDirectories();
    }//GEN-LAST:event_openDirectoriesMenuItemActionPerformed

    private void exportMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exportMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.export();
    }//GEN-LAST:event_exportMenuItemActionPerformed

    private void alignDatasMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alignDatasMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.alignDatas();
    }//GEN-LAST:event_alignDatasMenuItemActionPerformed

    private void closeDirectoriesMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeDirectoriesMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.closeDirectories();
    }//GEN-LAST:event_closeDirectoriesMenuItemActionPerformed

    private void printLadderMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_printLadderMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.printLadder();
    }//GEN-LAST:event_printLadderMenuItemActionPerformed

    private void printSampleMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_printSampleMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.printSample();
    }//GEN-LAST:event_printSampleMenuItemActionPerformed

    private void loadProjectMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadProjectMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.chooseStudyFile();
    }//GEN-LAST:event_loadProjectMenuItemActionPerformed

    private void saveProjectMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveProjectMenuItemActionPerformed
        // TODO add your handling code here:
        risaAligner.saveStudy();
    }//GEN-LAST:event_saveProjectMenuItemActionPerformed

    private void initLicenceMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_initLicenceMenuItemActionPerformed
        // TODO add your handling code here:
        reinitLicence();
    }//GEN-LAST:event_initLicenceMenuItemActionPerformed

    private void setLicenceMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_setLicenceMenuItemActionPerformed
        // TODO add your handling code here:
        setLicence();
    }//GEN-LAST:event_setLicenceMenuItemActionPerformed

    private void showLicenceMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showLicenceMenuItemActionPerformed
        // TODO add your handling code here:
        showLicence();
    }//GEN-LAST:event_showLicenceMenuItemActionPerformed

    private void formPropertyChange(java.beans.PropertyChangeEvent evt) {//GEN-FIRST:event_formPropertyChange
        // TODO add your handling code here:
        
    }//GEN-LAST:event_formPropertyChange
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem alignDatasMenuItem;
    private javax.swing.JMenuItem alignLadderMenuItem;
    private javax.swing.JMenuItem closeDirectoriesMenuItem;
    private javax.swing.JMenuItem closeDirectoryMenuItem;
    private javax.swing.JMenuItem exportMenuItem;
    private javax.swing.JMenuItem initLicenceMenuItem;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenu jMenu3;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPopupMenu.Separator jSeparator1;
    private javax.swing.JPopupMenu.Separator jSeparator2;
    private javax.swing.JPopupMenu.Separator jSeparator4;
    private javax.swing.JPopupMenu.Separator jSeparator5;
    private javax.swing.JMenuItem loadProjectMenuItem;
    private javax.swing.JMenuItem openDirectoriesMenuItem;
    private javax.swing.JMenuItem openDirectoryMenuItem;
    private javax.swing.JMenuItem printLadderMenuItem;
    private javax.swing.JMenu printMenu;
    private javax.swing.JMenuItem printSampleMenuItem;
    private javax.swing.JMenuItem saveProjectMenuItem;
    private javax.swing.JMenuItem setLicenceMenuItem;
    private javax.swing.JMenuItem showLicenceMenuItem;
    // End of variables declaration//GEN-END:variables
    
    @Override
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();
        if (source == alignLadderButton) {
            risaAligner.alignLadders();
        }
        else if (source == lockButton) {
            risaAligner.setLocked(lockButton.isSelected());
        }
        else if (source == alignDatasButton) {
            risaAligner.alignDatas();
        }
        else if (source == shadeButton) {
            if (shadeButton.isSelected()) {
                risaAligner.setSampleViewMode (RisaAligner.SHADE_MODE);
            }
            else {
                risaAligner.setSampleViewMode (RisaAligner.CURVES_MODE);
            }
        }
        else if (source == view0Button) {
            risaAligner.setViewMode(RisaAligner.VIEW_SAMPLE);
        }
        else if (source == view1Button) {
            risaAligner.setViewMode(RisaAligner.VIEW_LADDER_ALIGNED);
        }
        else if (source == view2Button) {
            risaAligner.setViewMode(RisaAligner.VIEW_DATAS_ALIGNED);
        }
        else if (source == exportButton) {
            risaAligner.export();
        }
    }
}
