/*
 * BanqueFileChooser.java
 *
 * Created on 1 juillet 2003, 17:35
 * 
 * Copyright 2015 Riccardo Scorretti, Olivier Fabregues, Sandrine Demaneche, Elisabeth Navarro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */
package risaaligner;

import java.io.File;
import java.util.*;
import javax.swing.*;
import javax.swing.filechooser.*;

/**
 *
 * @author  olivier fabregue
 */
public class LadderFileChooser extends javax.swing.JDialog {
    static final long serialVersionUID = 00L;
    String fileName="";
    Hashtable<String,ImageFilter> filterList;
    
    /** Creates new form BanqueFileChooser */
    public LadderFileChooser(java.awt.Frame parent, boolean modal,File directory) {
        super(parent, modal);
        initComponents();
        if (directory != null) {
            jFileChooser1.setCurrentDirectory(directory);
        }
        filterList = new Hashtable<String,ImageFilter>();
    }
    public void allowDirectory() {
        jFileChooser1.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    }
    public void allowOnlyDirectory() {
        jFileChooser1.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    }
    
    /** add type of files to explore
     * @param logiciel name of the software using this file's type
     * @param extension file extension for this file's type
     * @param descript tooltype for this file's type
     */    
    public void addType (String logiciel, String extension, String descript) {
        ImageFilter imageFilter = new ImageFilter(extension,descript);
        filterList.put(logiciel,imageFilter);
        jFileChooser1.addChoosableFileFilter(imageFilter);
    }
    /** add type of files to explore
     * @param logiciel name of the software using this file's type
     * @param extension file extension for this file's type
     * @param descript tooltype for this file's type
     */    
    public void addType (String logiciel,String prefix, String extension, String descript) {
        ImageFilter imageFilter = new ImageFilter(prefix,extension,descript);
        filterList.put(logiciel,imageFilter);
        jFileChooser1.addChoosableFileFilter(imageFilter);
    }
    
    /** choose a default selected type of files
     * @param filter name of the software to list
     */    
    public void chooseFilter (String filter) {
        jFileChooser1.setFileFilter(filterList.get(filter));
    }
        
    /** get selected file
     * @return name of selected file
     */    
    public File getFile(String message) {
        File file = null;
        int returnVal = jFileChooser1.showDialog(null,message);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            file = jFileChooser1.getSelectedFile();
        }
        return (file);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jFileChooser1 = new javax.swing.JFileChooser();

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        getContentPane().add(jFileChooser1, java.awt.BorderLayout.CENTER);

        pack();
    }//GEN-END:initComponents
    
            
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JFileChooser jFileChooser1;
    // End of variables declaration//GEN-END:variables
    
}
class ImageFilter  extends FileFilter{
    String prefix,extension,descript;
    public ImageFilter (String extension,String descript) {
        this.extension = extension;
        this.descript = descript;
    }
    
    public ImageFilter (String prefix,String extension,String descript) {
        this.prefix = prefix;
        this.extension = extension;
        this.descript = descript;
    }
    
    public boolean accept(File file) {
        if (file.isDirectory()) {
            return true;
        }
        String fileName = file.getName();
        int index = fileName.lastIndexOf('.');
        if (index >= 0) {
            String type = fileName.substring(index);
            if (type.equalsIgnoreCase(extension)) {
                if (prefix != null) {
                    if (fileName.startsWith(prefix)) return true;
                }
                else {
                    return(true);
                }
            }
        }
        return(false);
    }

    // The description of this filter
    public String getDescription() {
        return (descript);
    }
}
