/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * 
 * Copyright 2015 Riccardo Scorretti, Olivier Fabregues, Sandrine Demaneche, Elisabeth Navarro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */

/*
 * RisaAligner.java
 *
 * Created on 17 mars 2015, 09:42:10
 */
package risaaligner;

import java.io.FileNotFoundException;
import risaaligner.datas.Sample;
import risaaligner.datas.DataSet;
import java.io.IOException;
import ampere.graphic2d.Curve.Graphe;
import ampere.graphic2d.geom.GRectangleScreen;
import ampere.tui.TuiException;
import ampere.util.file.FileUtil;
import ampere.util.primitives.Table;
import ampere.util.primitives.TableElementExistException;
import ampere.util.io.OutPanel;
import java.awt.BasicStroke;
import java.awt.Color;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.prefs.Preferences;
import javax.swing.JOptionPane;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;
import risaaligner.datas.SampleReader;
import risaaligner.datas.Spot;
import risaaligner.datas.SpotLine;
import risaaligner.datas.TuiSampleReader;
import risaaligner.graphic.GuiLadder;
import risaaligner.graphic.GuiSample;
import risaaligner.graphic.ShadeSample;

/**
 *
 * @author olivier
 */
public class RisaAligner extends javax.swing.JPanel {

    int state = 0;

    public static int STATE_NOTHING = 0;
    public static int STATE_SAMPLE = 1;
    public static int STATE_ALIGNABLE = 2;
    public static int STATE_LADDER_ALIGNED = 3;
    
    int viewAlignMode = 0;
    
    public static int VIEW_SAMPLE = 0;
    public static int VIEW_LADDER_ALIGNED = 1;
    public static int VIEW_DATAS_ALIGNED = 2;
    
    int sampleViewMode = 0;
    
    public static int CURVES_MODE = 0;
    public static int SHADE_MODE = 1;
    
    final String separator = new String(new char[]{'\t'});
    final String newLine = new String(new char[]{'\n'});
    
    public static RisaAligner risaAligner;
    
    public Table<String,Sample> tab_samples;
    public Table<String,TabPanel> tab_tabPanels;
    
    int selectedSampleIndex = -1;
    GuiLadder selectedGuiLadder = null;
    Sample selectedSample = null;
    
    public Table<String,GuiLadder> tab_guiLadders;
    GuiSample guiSample;
    ShadeSample shadeSample;
    
    ShadeSample imageSample;

    Sample refSample;
    
    Preferences userPrefs = null;
    Graphe graphe = null;
    GRectangleScreen[] gSpots;
    
    public File saveFile = null;
    public File projectFile = null;
    File alignedFile = null;
    int readerType = -1;
    
    BasicStroke lineStroke = new BasicStroke(2.f);
    Color[] curveColors = {Color.BLUE,Color.CYAN,Color.GREEN,Color.MAGENTA,Color.ORANGE,Color.PINK,Color.RED,Color.YELLOW};
    int colorIndice = 0;
    
    DataSet generalDataSet = null;
    boolean shadeVisible = false;

    public boolean locked;

    private javax.swing.JTabbedPane jTabbedPane1;
    OutPanel outPanel;
    
    boolean loadingProject=false;
    
    boolean licenceOk = false;
    
    MainInterface mainProgram;

    /** Creates new form RisaAligner */
    public RisaAligner(MainInterface mainProgram) {
        this.mainProgram = mainProgram;
        risaAligner = this;
        
        tab_guiLadders = new Table<String,GuiLadder>();
        tab_tabPanels = new Table<String,TabPanel>();
        tab_samples = new Table<String,Sample>();

        initComponents();
        
        /*
         * creation de la sortie standard
         */
        outPanel = new OutPanel();
        outPanel.setPreferredSize(new java.awt.Dimension(10, 50));
        jSplitPane0.setBottomComponent(outPanel);

        /*
         * initialisation de la taille avec la taille par defaut 
         */
        GuiLadder guiLadder = new GuiLadder(null);
        
        /*
         * construction du jTabbedPane
         * le jTabbedPane est changé car il y a un bug.
         * Le TabbedPane de Sun recalcule mal la taille des onglets
         * et on ne peut pas resizer correctement la fenetre
         */
        
        jTabbedPane1 = new javax.swing.JTabbedPane();
        /*
        jTabbedPane1 = new javax.swing.JTabbedPane() {
            public Dimension getMinimumSize() {
                Dimension dim = calculateSize(true);
                return dim;
            }
        };
         * 
         */
        jTabbedPane1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jTabbedPane1StateChanged(evt);
            }
        });
        jPanel2.add(jTabbedPane1, java.awt.BorderLayout.CENTER);
        jTabbedPane1.setPreferredSize(guiLadder.getPreferredSize());
        guiLadder = null;
        
        /*
         * construction des composants du bas
         */
        guiSample = new GuiSample();

        shadeSample = new ShadeSample();
        
        /*
         * initialisation a la taille par defaut
         */
        jPanel1.setPreferredSize(guiSample.getPreferredSize());

        /*
         * creation du fichier de sauvegarde
         */
        saveFile = new File ("~saveFile.jof");

        /*
         * verification de la licence
         */
        licenceOk = mainProgram.verifyLicence();
        mainProgram.updateButtons(licenceOk,locked,STATE_NOTHING, null);

        /*
         * restauration eventuelle de la sesson precedente
         */
        if (licenceOk) {
            /*
             * demande de confirmation de restauration si il existe
             */
            if (saveFile.exists()) {

                int n = JOptionPane.showConfirmDialog(this,"RisaAligner has abnormally shutdowned. Restore previous session ?", "", JOptionPane.YES_NO_OPTION);
                if (n == JOptionPane.YES_OPTION) {
                    restore(saveFile);
                }
                else {
                    saveFile.delete();
                }
            }
        }
    }

    public void loadConfig(String projectFileName) {
        if (projectFileName != null) projectFile = new File(projectFileName);
    }
    private void saveConfig() {
        mainProgram.saveConfig(projectFile.getPath());
    }
    

    private void restore(File file) {
        
        File newFile;
        try {
            newFile = FileUtil.copyFile(file, "~saveFile1.jof");
            loadStudy(newFile);
            scaleAndRepaint();
            updateButtons();
            saveStudy(saveFile);
        } 
        catch (FileNotFoundException ex) {
            writeError("> File not found"+'\n');
        } 
        catch (IOException ex) {
            writeError("> Reading error"+'\n');
        }
        
    }
    public void chooseStudyFile() {
        /*
        if (projectFile == null) {
            projectFile = new File("C:\\travail\\labo\\recherche\\risa_aligner\\datas\\donnees-brutes");
        }
         * 
         */
        LadderFileChooser fileChooser = new LadderFileChooser(null,true,projectFile);
        //fileChooser.allowOnlyDirectory();
        fileChooser.addType ("project file", ".jof", "fichiers d'études");
        File studyFile = fileChooser.getFile ("Ouvrir");
        if (studyFile != null) {
            writeMsg("> Load project from "+studyFile.getPath()+'\n');
            loadStudy (studyFile);
            scaleAndRepaint();
            updateButtons();
        }
    }
    
    private void loadStudy(File file) {
        try {
            BufferedReader br = null;
            br = new BufferedReader(new FileReader(file));

            String line = null;

            while ((line = br.readLine()) != null) {

                if (line != null && line.length()!=0) {

                    String[] fields = line.split(";");

                    // debut de la lecture des données
                    if (fields[0].equals("loadDirectories")) {
                        String projectPath = fields[1].trim();
                        File projectFile = new File (projectPath);
                        int type = Integer.valueOf(fields[2].trim()).intValue();
                        loadDirectories(projectFile,type);
                    }
                    else if (fields[0].equals("closeDirectories")) {
                        closeDirectories();
                    }
                    else if (fields[0].equals("setProject")) {
                        String projectPath = fields[1].trim();
                        projectFile = new File(projectPath);
                    }
                    else if (fields[0].equals("loadDirectory")) {
                        String directoryPath = fields[1].trim();
                        File directoryFile = new File (directoryPath);
                        readerType = Integer.valueOf(fields[2].trim()).intValue();
                        loadDirectory(directoryFile,readerType);
                        _setVisibleSample(tab_samples.size()-1);
                    }
                    else if (fields[0].equals("closeDirectory")) {
                        String directoryName = fields[1].trim();
                        closeDirectory(directoryName);
                    }
                    else if (fields[0].equals("setSampleRef")) {
                        if (fields.length>1) {
                            String sampleName = fields[1].trim();
                            setSampleRef(sampleName);
                        }
                        else {
                            setSampleRef(null);
                        }
                    }
                    else if (fields[0].equals("setLevel")) {
                        String sampleName = fields[1].trim();
                        double level = Double.valueOf(fields[2].trim()).doubleValue();
                        setLevel (sampleName,level);
                    }
                    else if (fields[0].equals("setSpotState")) {
                        String sampleName = fields[1].trim();
                        int type = Integer.valueOf(fields[2].trim()).intValue();
                        int index = Integer.valueOf(fields[3].trim()).intValue();
                        int state = Integer.valueOf(fields[4].trim()).intValue();
                        setSpotState (sampleName,type,index,state);
                    }
                    else if (fields[0].equals("alignLadders")) {
                        alignLadders();
                    }
                    else if (fields[0].equals("removeLadderAlignment")) {
                        removeLadderAlignment();
                    }
                    else if (fields[0].equals("createSpotLineAuto")) {
                        double xValue = Double.valueOf(fields[1].trim()).doubleValue();
                        createSpotLineAuto (xValue);
                    }
                    else if (fields[0].equals("createSpotLine")) {
                        String sampleName = fields[1].trim();
                        int dataSetIndex = Integer.valueOf(fields[2].trim()).intValue();
                        int index = Integer.valueOf(fields[3].trim()).intValue();
                        int xIndex = Integer.valueOf(fields[4].trim()).intValue();
                        int state = Integer.valueOf(fields[5].trim()).intValue();
                        createSpotLine (sampleName,dataSetIndex,index,xIndex,state,true);
                    }
                    else if (fields[0].equals("removeSpotLine")) {
                        int index = Integer.valueOf(fields[1].trim()).intValue();
                        removeSpotLine(index);
                    }
                    else if (fields[0].equals("changeSpotLineState")) {
                        int noDataSet = Integer.valueOf(fields[1].trim()).intValue();
                        int index = Integer.valueOf(fields[2].trim()).intValue();
                        changeSpotLineState (noDataSet, index);
                    }
                    else if (fields[0].equals("moveSpotLine")) {
                        int noDataSet = Integer.valueOf(fields[1].trim()).intValue();
                        int index = Integer.valueOf(fields[2].trim()).intValue();
                        double value = Double.valueOf(fields[3].trim()).doubleValue();
                        moveFinalSpotLine (noDataSet,index,value);
                    }
                    else if (fields[0].equals("alignDatas")) {
                        alignDatas ();
                    }
                    else if (fields[0].equals("alignSampleDatas")) {
                        String sampleName = fields[1].trim();
                        alignDatas (sampleName);
                    }
                    else if (fields[0].equals("removeAllDatasAlignment")) {
                        removeAllDatasAlignment();
                    }
                    else if (fields[0].equals("removeDatasAlignment")) {
                        String sampleName = fields[1].trim();
                        removeDatasAlignment(sampleName);
                    }
                    else if (fields[0].equals("setVisibleSample")) {
                        int sampleIndex = Integer.valueOf(fields[1].trim()).intValue();
                        setVisibleSample(sampleIndex);
                    }
                    else if (fields[0].equals("setViewMode")) {
                        int viewMode = Integer.valueOf(fields[1].trim()).intValue();
                        setViewMode(viewMode);
                    }
                    else if (fields[0].equals("setSampleViewMode")) {
                        int sampleViewMode = Integer.valueOf(fields[1].trim()).intValue();
                        setSampleViewMode(sampleViewMode);
                    }
                        
                }
           }
        }
        
        catch (Exception ex) {
        }
    }
    public void saveStudy() {
        LadderFileChooser fileChooser = new LadderFileChooser(null,true,projectFile);
        //fileChooser.allowOnlyDirectory();
        fileChooser.addType ("project file", ".jof", "fichiers d'études");
        File studyFile = fileChooser.getFile ("Save");
        if (studyFile == null) return;
        String fileName = studyFile.getName();
        int index = fileName.lastIndexOf(".");
        if (index >=0) {
            fileName= fileName.substring(0,index);
        }
        fileName = fileName.concat(".jof");
        studyFile = new File(studyFile.getParent(),fileName);
        if (studyFile != null) {
            if (studyFile.exists()) {
                int n = JOptionPane.showConfirmDialog(this,"File exists. Sure to overwrite ?", "", JOptionPane.YES_NO_OPTION);
                if (n == JOptionPane.NO_OPTION) {
                    return;
                }
            }
            writeMsg("> Save project in "+studyFile.getPath()+'\n');
            saveStudy (studyFile);
        }
    }
    private void saveStudy(File file) {
        FileWriter writer;
        file.delete();
        try {
            writer = new FileWriter(file,true);
            BufferedWriter w = new BufferedWriter(writer);
            
            if (projectFile != null) {
                String string = "setProject"+";"+projectFile.getAbsolutePath();
                w.write (string+newLine);
            }
            for (int i=0;i<tab_samples.size();i++) {
                Sample sample = tab_samples.getObject(i);
                String string = "loadDirectory"+";"+sample.repPath+";"+readerType;
                w.write (string+newLine);
            }
            if (refSample != null) {
                String string = "setSampleRef"+";"+refSample.repName;
                w.write (string+newLine);
            }
            for (int i=0;i<tab_samples.size();i++) {
                Sample sample = tab_samples.getObject(i);
                for (int j=0;j< sample.spots.length;j++) {
                    Spot spot = sample.spots[j];
                    if (spot.state != Spot.AUTO) {
                        String string = "setSpotState"+";"+sample.repName+";"+String.valueOf(spot.type)+";"+String.valueOf(spot.index)+";"+String.valueOf(spot.state);
                        w.write (string+newLine);
                    }
                }
                if (sample.refSpots != null) {
                    for (int j=0;j< sample.refSpots.length;j++) {
                        Spot spot = sample.refSpots[j];
                        if (spot.state != Spot.AUTO) {
                            String string = "setSpotState"+";"+sample.repName+";"+String.valueOf(spot.type)+";"+String.valueOf(spot.index)+";"+String.valueOf(spot.state);
                            w.write (string+newLine);
                        }
                    }
                }
            }
            for (int i=0;i<tab_samples.size();i++) {
                Sample sample = tab_samples.getObject(i);
                String string = "setLevel"+";"+sample.repName+";"+String.valueOf(sample.level);
                w.write (string+newLine);
            }
                    
            if (state >= STATE_LADDER_ALIGNED) {
                String string = "alignLadders";
                w.write (string+newLine);
            }

            for (int i=0;i<tab_samples.size();i++) {
                Sample sample = tab_samples.getObject(i);
                DataSet dataSet = sample.ladderDataSet;
                for (int k=0;k<dataSet.spotLines.length;k++) {
                    SpotLine spotLine = dataSet.spotLines[k];
                    String string = "createSpotLine"+";"+sample.repName+";-1;"+String.valueOf(spotLine.index)+";"+String.valueOf(spotLine.xIndex0)+";"+String.valueOf(spotLine.state);
                    w.write (string+newLine);
                }
  
                for (int j=0;j<sample.dataSets.length;j++) {
                    dataSet = sample.dataSets[j];
                    for (int k=0;k<dataSet.spotLines.length;k++) {
                        SpotLine spotLine = dataSet.spotLines[k];
                        String string = "createSpotLine"+";"+sample.repName+";"+String.valueOf(j)+";"+String.valueOf(spotLine.index)+";"+String.valueOf(spotLine.xIndex0)+";"+String.valueOf(spotLine.state);
                        w.write (string+newLine);
                    }
                }
                
                if (sample.isDatasAligned()){
                    String string = "alignSampleDatas"+";"+sample.repName;
                    w.write (string+newLine);
                }
            }
                   
            if (selectedSampleIndex >= 0) {
                String string = "setVisibleSample"+";"+selectedSampleIndex;
                w.write (string+newLine);
            }
            
            String string = "setSampleViewMode"+";"+sampleViewMode;
            w.write (string+newLine);
            
            w.close();
        } catch (IOException ex) {
        }
    }

    private void jTabbedPane1StateChanged(javax.swing.event.ChangeEvent evt) {                                          
        // TODO add your handling code here:
        if (loadingProject) return;

        int index = jTabbedPane1.getSelectedIndex();
        setVisibleSample(index);
    }                                         

    public void writeInSaveFile (String string) {
        FileWriter writer;
        try {
            writer = new FileWriter(saveFile,true);
            BufferedWriter w = new BufferedWriter(writer);
            w.write (string+newLine);
            w.close();
        } catch (IOException ex) {
        }
    }
    
    public void chooseDirectories() {
        /*
        if (projectFile == null) {
            projectFile = new File("C:\\travail\\labo\\recherche\\risa_aligner\\datas\\donnees-brutes");
        }
         * 
         */
        LadderFileChooser fileChooser = new LadderFileChooser(null,true,projectFile);
        fileChooser.allowOnlyDirectory();
        fileChooser.addType ("Risa Aligner", ".csv", "fichiers de données");
        File file = fileChooser.getFile ("Ouvrir");
        if (file == null) return;
        
        try {
            File[] files = file.listFiles();
            /*
             * recherche des types de fichiers présents dans le repertoire
             */
            int type = getReaderType(files);
            if (type <0) return;
            loadDirectories (file,type);
            _setVisibleSample(0);
            scaleAndRepaint();
            updateButtons();

            writeMsg("> Load all directories "+file.getPath()+'\n');
        } 
        catch (TuiException ex) {
        }
    }
    
  
    
    /*
     * appellé par la commande de script "loadDirectories" et par la methode chooseDirectories
     */
    private void loadDirectories(File directory,int type) {
        
        if (directory == null) return;

        /*
         * initialisation du repertoire de base au parent
         */
        projectFile = directory;

        /*
         * desactivation du listener sur le changement de selection du tabPane
         */
        loadingProject = true;


        /*
         * recherche des repertoires de fichiers de données dans ce repertoire
         */
        File[] files = directory.listFiles();
        for (int i=0;i<files.length;i++) {

            /*
             * lecture du selectedSample
             */
            File file = files[i];
            openDirectory(file,type);
        }
        
        /*
         * selection du premier onglet
         */
        if (tab_tabPanels.size()>0) {
            state = STATE_SAMPLE;
        }

        /*
         * reactivation du listener sur le changement de selection du tabPane
         */
        loadingProject = false;
        
        String string = "loadDirectories"+";"+directory.getAbsolutePath()+";"+type;
        writeInSaveFile (string );
        
    }

    public void closeDirectories() {
        
        removeAllSamplesFromGui();
        writeMsg("> Close all directories"+projectFile.getPath()+'\n');
        writeInSaveFile ("closeDirectories");
        
    }

    public void chooseDirectory() {
        if (projectFile == null) {
            //projectFile = new File("C:\\travail\\labo\\recherche\\risa_aligner\\datas\\donnees-brutes");
        }
        LadderFileChooser fileChooser = new LadderFileChooser(null,true,projectFile);
        fileChooser.allowOnlyDirectory();
        File file = fileChooser.getFile ("Ouvrir");
        if (file == null) return;
        writeMsg("> Load directory "+file.getPath()+'\n');

        try {
            /*
             * recherche des types de fichiers présents dans le repertoire
             */
            File[] files = new File[1];
            files[0] = file;
            int type = getReaderType(files);
            if (type <0) return;
            loadDirectory (file,type);
            _setVisibleSample(tab_samples.size()-1);
            scaleAndRepaint();
            updateButtons();
            writeMsg("> Load all directories "+file.getPath()+'\n');
        } 
        catch (TuiException ex) {
        }
    }

    public int getReaderType(File[] files) throws TuiException {
        readerType = -1;
        boolean b1 = false;
        boolean b2 = false;
        boolean b3 = false;
        boolean b4 = false;
        boolean b5 = false;
        for (int i=0;i<files.length;i++) {
            SampleReader sampleReader = new  SampleReader(files[i]);
            if (sampleReader.existNormal()) b1 = true;
            if (sampleReader.existAligned()) b2 = true;;
            if (sampleReader.existAligned2()) b3 = true;
            if (sampleReader.existAlignedNegat()) b4 = true;
            if (sampleReader.existAligned2Negat()) b5 = true;
        }
        int nbTypes = 0;
        if (b1) {
            readerType = 0;
            nbTypes++;
        }
        if (b4) {
            readerType = 3;
            nbTypes++;
        }
        if (b5) {
            readerType = 4;
            nbTypes++;
        }
        if (b2) {
            readerType = 1;
            nbTypes++;
        }
        if (b3) {
            readerType = 2;
            nbTypes++;
        }
        if (nbTypes >1) {
            TuiSampleReader tuiReader  = new TuiSampleReader(null, "Type de données à lire","Type de données",
                                                        b1,b2,b3,b4,b5,readerType);
            readerType = tuiReader.getType();
        }
        return readerType;
    }
    
    /*
     * appellé par la commande de script "loadDirectory" et par la methode chooseDirectory
     */
    public void loadDirectory (File file,int type) {
        /*
         * initialisation du repertoire de base au parent
         */
        projectFile = file.getParentFile();
        
        /*
         * desactivation du listener sur le changement de selection du tabPane
         */
        loadingProject = true;

        /*
         * lecture du repertoire
         */
        openDirectory(file,type);
        
        /*
         * selection du dernier onglet ouvert
         */
        state = STATE_SAMPLE;
        //_setVisibleSample(tab_samples.size()-1);
        //updateButtons();
        
        /*
         * reactivation du listener sur le changement de selection du tabPane
         */
        loadingProject = false;
        
        String string = "loadDirectory" +";"+ file.getAbsolutePath()+";"+type;
        writeInSaveFile (string);
    }
    
    /*
     * appelle par la methode loadDirectory et par la methode loadDirectories
     */
    public void openDirectory(File file,int type) {
        
        try {
            /*
             * lecture du Sample
             */
            SampleReader sampleReader = new  SampleReader(file);
            Sample sample = sampleReader.getSample(type);
            
            /*
             * enregistrement du selectedSample dans l interface
             */
            if (sample != null) addSampleToGui(sample);
        } 
        catch (TableElementExistException ex) {
            writeMsg("> Puce with same name already exists"+'\n');
        }    
    }

    
    private void addSampleToGui(Sample sample) throws TableElementExistException {

        /*
         * creation du graphique associé a ce repertoire
         */ 
        GuiLadder guiLadder = new GuiLadder(sample);
        tab_guiLadders.add(sample.repName, guiLadder);
        tab_samples.add(sample.repName, sample);
        
        /*
         * ajout de ce graphique dans un nouvel onglet
         */
        jTabbedPane1.addTab (sample.repName,guiLadder);

        /*
         * modification de l'onglet
         */
        TabPanel tabPanel = new TabPanel(sample.repName,sample);
        jTabbedPane1.setTabComponentAt(jTabbedPane1.getTabCount()-1, tabPanel);
        jTabbedPane1.setSelectedComponent(guiLadder);
        
        /*
         * memorisation de cet onglet
         */
        tab_tabPanels.add(sample.repName, tabPanel);
        
        
    }
    public void closeCurrentDirectory() {
        String dirName = jTabbedPane1.getTitleAt(jTabbedPane1.getSelectedIndex());
        closeDirectory(dirName);
        writeMsg("> Close directory "+dirName+'\n');
    }
    
    private void closeDirectory(String dirName) {
        removeSampleFromGui(dirName);
        String string = "closeDirectory"+";"+ dirName;
        writeInSaveFile (string );
    }
    
    private void removeSampleFromGui(String sampleName)  {

        int index = tab_tabPanels.getIndexFromKey(sampleName);
        jTabbedPane1.remove(index);
        
        GuiLadder guiLadder = tab_guiLadders.removeObject(index);
        guiLadder = null;
        
        Sample sample = tab_samples.removeObject(index);
        sample = null;
        
        TabPanel tabPanel = tab_tabPanels.removeObject(index);
        tabPanel = null;

        /*
         * mise a jour eventuelle du selectedSample affiché
         */

        int toSelect = selectedSampleIndex;
        if (selectedSampleIndex > index) {
            toSelect--;
        }
        else if (selectedSampleIndex == index) {
            toSelect--;
            if (toSelect<0 && tab_samples.size()>0) toSelect = 0;
            _setVisibleSample(toSelect);
            scaleAndRepaint();
            updateButtons();

        }

        if (tab_guiLadders.size()==0) {
            removeAllSamplesFromGui();
        }
    }
    
    void removeAllSamplesFromGui() {

        jTabbedPane1.removeAll();
        jPanel1.removeAll();
        
        _setVisibleSample(-1);
        
        validate();
        repaint();
        
        tab_guiLadders.clear();
        tab_samples.clear();
        tab_tabPanels.clear();
        

        state = STATE_NOTHING;
        locked = false;
        updateButtons();
    }

    public void updateButtons () {
        mainProgram.updateButtons(licenceOk,locked, state, tab_samples.getObject(selectedSampleIndex));
    }

    /*
    public void shadeIsClosing() {
        this.shadeVisible = false;
        shadeButton.setSelected(shadeVisible);
    }
     * 
     */

    public void quit() {
        
        saveFile.delete();
        saveConfig();
        System.exit(0);
    }

    /*
     * retracage de l onglet selectionné et du selectedSample 
     */
    public void scaleAndRepaint() {
        if (selectedSampleIndex>=0) tab_guiLadders.getObject(selectedSampleIndex).scaleAndRepaint();
        if (sampleViewMode == 0) {
            guiSample.scaleAndRepaint();
        }
        else if (sampleViewMode == 1){
            shadeSample.scaleAndRepaint();
        }
    }
    
    /*
     * recalcul des min et max sur les ladders de chaque selectedSample
     * par exemple si la reference a changé
     */
    public void updateLaddersMinMax() {
        for (int i=0;i<tab_guiLadders.size();i++) {
            GuiLadder guiLadder = tab_guiLadders.getObject(i);
            guiLadder.updateMinMax();
        }
    }
    
    public void setSampleRef(String  sampleName) {
        if (locked) return;
        
        Sample refSample = tab_samples.getObject(sampleName);
        
        /*
         * deselection du selectedSample de refernce
         */
        if (refSample == null || refSample == this.refSample) {

            for (int i=0;i<tab_samples.size();i++) {
                Sample sample = tab_samples.getObject(i);
                sample.removeRefSample();

                GuiLadder guiLadder = tab_guiLadders.getObject(i);
                guiLadder.removeRefSample(); 

                TabPanel tabPanel = tab_tabPanels.getObject(i);
                tabPanel.setSelected(false);
            }

            this.refSample = null;
            state = STATE_SAMPLE;
            writeMsg("> Unset reference "+'\n');
            writeInSaveFile ("setSampleRef" );
        }

        /*
         * selection du selectedSample reference
         */
        else {
            
            for (int i=0;i<tab_guiLadders.size();i++) {
                Sample sample = tab_samples.getObject(i);
                sample.setRefSample(refSample);
                
                GuiLadder guiLadder = tab_guiLadders.getObject(i);
                guiLadder.setRefSample(refSample);

                TabPanel tabPanel = tab_tabPanels.getObject(i);
                if (guiLadder.sample.isReference()) {
                    tabPanel.setSelected(true);
                }
                else {
                    tabPanel.setSelected(false);
                }
            }
            this.refSample = refSample;
            
            state = STATE_ALIGNABLE;
            writeMsg("> Set reference "+refSample.repName+'\n');
            String string = "setSampleRef"+";" + sampleName;
            writeInSaveFile (string);
        }
        _setViewMode (VIEW_SAMPLE);
        updateButtons();
        updateLaddersMinMax();
        scaleAndRepaint();

    }

    public void setLevel(String sampleName,double level) {
        Sample sample = tab_samples.getObject(sampleName);
        GuiLadder guiLadder = tab_guiLadders.getObject(sampleName);
        _removeLadderAlignment();
        if (sample != null) {
            sample.setLevel(level);
        }
        if (guiLadder != null) {
            guiLadder.updateLevel();
        }
        if (selectedSample != null) {
            sample.setAlignable(guiLadder.isAlignable());
        }
    }
    
    public void setLevel(double level) {
        _removeLadderAlignment();
        if (selectedSample != null) {
            selectedSample.setLevel(level);
        }
        if (selectedGuiLadder != null) {
            selectedGuiLadder.updateLevel();
        }
        if (selectedSample != null) {
            selectedSample.setAlignable(selectedGuiLadder.isAlignable());
        }
        _setViewMode(VIEW_SAMPLE);
        updateButtons();
    }
    public void setFinalLevel(double level) {
        String string = "setLevel"+";" + selectedSample.repName +";"+ String.valueOf(level);
        writeInSaveFile(string);
    }
    
    public void changeSpotState (int spotType,int spotIndex) {
        int spotState = state;
        _removeLadderAlignment();

        if (selectedSample != null ) {
            spotState = selectedSample.changeSpotState(spotType,spotIndex);
        }
        if (selectedGuiLadder != null) {
            selectedGuiLadder.updateSpotState(spotType,spotIndex);
        }
        if (selectedSample != null) {
            selectedSample.setAlignable(selectedGuiLadder.isAlignable());
        }
        String string = "setSpotState"+";" + selectedSample.repName +";"+ String.valueOf(spotType)+";"+ String.valueOf(spotIndex)+";"+String.valueOf(spotState);
        writeInSaveFile(string);
    }

    public void setSpotState (String sampleName,int spotType,int spotIndex,int state) {
        _removeLadderAlignment();
        Sample sample = tab_samples.getObject(sampleName);
        GuiLadder guiLadder = tab_guiLadders.getObject(sampleName);
         if (sample != null) {
            sample.setSpotState(spotType,spotIndex,state);
            guiLadder.updateSpotState(spotType,spotIndex);
            sample.setAlignable(guiLadder.isAlignable());

            String string = "setSpotState"+";" + sample.repName +";"+ String.valueOf(spotType)+";"+ String.valueOf(spotIndex)+";"+String.valueOf(state);
            writeInSaveFile(string);
        }
    }
    
    /*
     * alignement des ladder qui sont alignables
     */
    public void alignLadders() {
        _removeLadderAlignment();

        try {
            for (int i=0;i<tab_guiLadders.size();i++) {
                GuiLadder guiSample = tab_guiLadders.getObject(i);
                if (guiSample.isAlignable()) {
                    PolynomialSplineFunction psf = guiSample.getAlignFunction();
                    Sample sample = guiSample.sample;
                    sample.alignLadder(psf);
                } 
            }
            state = STATE_LADDER_ALIGNED;
            writeMsg("> Align ladders with reference ladder"+'\n');
            writeInSaveFile ("alignLadders");
            
            _setViewMode(VIEW_LADDER_ALIGNED);
            setLocked(true);
            updateButtons();
        }
        catch (Exception e) {
            writeMsg("> Error alignment " + e.getMessage()+'\n');
        }
    }
    
    /*
     * reinitialisation du lock
     */
    /*
    public void setLock(boolean lock) {
        this.lock = lock;
        updateButtons();
    }
     * 
     */
    
    /*
     * reinitialisation de l alignement des ladder
     */
    public void removeLadderAlignment() {
        _removeLadderAlignment();
        _setViewMode(VIEW_SAMPLE);
        updateButtons();
        
        writeMsg("> Remove ladders alignment"+'\n');
        writeInSaveFile ("removeLadderAlignment");
    }
    public void _removeLadderAlignment() {
        _removeAllDatasAlignment();

        for (int i=0;i<tab_guiLadders.size();i++) {
            GuiLadder guiSample = tab_guiLadders.getObject(i);
            Sample sample = guiSample.sample;
            sample.removeAlignLadder();
        }
        state = STATE_ALIGNABLE;
    }
    
    public void createSpotLineAuto (double xValue){
        if (selectedSample != null) {
            _removeDatasAlignment(selectedSample);
            selectedSample.createSpotLineAuto(xValue,SpotLine.NORMAL,SpotLine.VISIBLE);
        }
        if (selectedGuiLadder != null) {
            shadeSample.createSpotLines();
        }
        _setViewMode(VIEW_LADDER_ALIGNED);
        updateButtons();
        
        String string = "createSpotLineAuto"+";"+String.valueOf(xValue).trim();
        writeInSaveFile (string);
    }
    public void createSpotLine (String sampleName,int dataSetIndex,int spotLineIndex,int xIndex, int state, boolean visible){
        Sample sample = tab_samples.getObject(sampleName);
        if (sample != null) {
            sample.createSpotLine(dataSetIndex,spotLineIndex,xIndex,state,visible);
        }
        
        String string = "createSpotLine"+";"+sampleName+";"+String.valueOf(dataSetIndex).trim()+";"+String.valueOf(spotLineIndex).trim()+";"+String.valueOf(xIndex).trim()+";"+state;
        writeInSaveFile (string);
    }
    public void removeSpotLine(int index) {
        if (selectedSample != null) {
            _removeDatasAlignment(selectedSample);
            selectedSample.removeSpotLine(index);
        }
        if (selectedGuiLadder != null) {
            shadeSample.createSpotLines();
        }
        _setViewMode(VIEW_LADDER_ALIGNED);
        updateButtons();
        
        String string = "removeSpotLine"+";"+index;
        writeInSaveFile (string);
    }
    public void changeSpotLineState (int noDataSet, int index) {
        if (selectedSample != null) {
            _removeDatasAlignment(selectedSample);
            selectedSample.changeSpotLineState(noDataSet,index);
        }
        if (selectedGuiLadder != null) {
            shadeSample.updateSpotLines();
        }
        _setViewMode(VIEW_LADDER_ALIGNED);
        updateButtons();
        
        String string = "changeSpotLineState"+";"+noDataSet+";"+index;
        writeInSaveFile (string);
    }
    public void moveSpotLine (int noDataSet,int index, double value) {
        if (selectedSample != null) {
            _removeDatasAlignment(selectedSample);
            selectedSample.moveSpotLine(noDataSet,index,value);
        }
        if (selectedGuiLadder != null) {
            shadeSample.updateSpotLines();
        }
        _setViewMode(VIEW_LADDER_ALIGNED);
        updateButtons();
    }
    public void moveFinalSpotLine (int noDataSet,int index, double value) {
        if (selectedSample != null) {
            _removeDatasAlignment(selectedSample);
            selectedSample.moveSpotLine(noDataSet,index,value);
        }
        if (selectedGuiLadder != null) {
            shadeSample.updateSpotLines();
        }
        _setViewMode(VIEW_LADDER_ALIGNED);
        updateButtons();
        String string = "moveSpotLine"+";"+noDataSet+";"+index+";"+value;
        writeInSaveFile (string);
    }
            
    
    /*
     * alignement secondaire du selectedSample selectionne
     */
    public void alignDatas () {
        Sample sample = tab_samples.getObject(selectedSampleIndex);
        alignSampleDatas(sample);
    }
    private void alignDatas (String sampleName) {
        Sample sample = tab_samples.getObject(sampleName);
        alignSampleDatas(sample);
    }

    private void alignSampleDatas (Sample sample) {
        try {
            _removeDatasAlignment(sample);
            sample.alignDatas();
            
            _setViewMode(VIEW_DATAS_ALIGNED);
            updateButtons();

            writeMsg("> Align datas from sample "+sample.repName+'\n');
            writeInSaveFile ("alignDatas");
        }
        catch (Exception e) {
            writeMsg("> Error alignment " + e.getMessage()+'\n');
        }


    }
    private void removeAllDatasAlignment() {
        _removeAllDatasAlignment();
        _setViewMode(VIEW_LADDER_ALIGNED);
        updateButtons();
        
        writeInSaveFile ("removeAllDatasAlignment");
    }
    private void removeDatasAlignment(String sampleName) {
        Sample sample = tab_samples.getObject(sampleName);

        _removeDatasAlignment(sample);
        _setViewMode(VIEW_LADDER_ALIGNED);
        updateButtons();
        
        String string = "removeDatasAlignment"+";"+sampleName;
        writeInSaveFile (string);
    }
    private void _removeAllDatasAlignment() {
        for (int i=0;i<tab_samples.size();i++) {
            Sample sample = tab_samples.getObject(i);
             _removeDatasAlignment(sample);
        }
    }
    private void _removeDatasAlignment(Sample sample) {
        sample.removeAlignDatas();
    }
    public void setLocked(boolean selected) {
        locked = selected;
        updateButtons();
    }
    public void setViewMode (int viewAlignMode) {
        _setViewMode(viewAlignMode);
        updateButtons();
        String string = "setViewMode"+";"+viewAlignMode;
        writeInSaveFile (string);
    }
    public void _setViewMode (int viewAlignMode) {
        if (this.viewAlignMode != viewAlignMode) {
            this.viewAlignMode = viewAlignMode;
            
            /*
             * mise a jour eventuelle du ladder
             */
            int sampleViewAlignMode = selectedSample.getBestViewFor(viewAlignMode);
            if (sampleViewAlignMode != selectedSample.viewMode) {
                selectedSample.viewMode = sampleViewAlignMode;
                selectedGuiLadder.viewAlignedDatas(sampleViewAlignMode);
                
                shadeSample.viewAlignedDatas(sampleViewAlignMode);
                guiSample.viewAlignedDatas(sampleViewAlignMode);
            }
            scaleAndRepaint();
        }
    }
    public void setSampleViewMode (int sampleViewMode) {
        _setSampleViewMode(sampleViewMode);
        scaleAndRepaint();
        updateButtons();

        String string = "setSampleViewMode"+";"+String.valueOf(sampleViewMode);
        writeInSaveFile (string);
    }
    public void _setSampleViewMode (int sampleViewMode) {
        if (this.sampleViewMode != sampleViewMode) {
            this.sampleViewMode = sampleViewMode;

            /*
             * mise a jour eventuelle des datas
             */
            jPanel1.removeAll();

            if (sampleViewMode==CURVES_MODE) {
                jPanel1.add(guiSample, java.awt.BorderLayout.CENTER);
            }
            else if (sampleViewMode == SHADE_MODE) {
                jPanel1.add(shadeSample, java.awt.BorderLayout.CENTER);
            }

            jPanel1.validate();
        }
    }
    /*
     * change visible sample when 
     * - change by tabbedPane 
     * - open or close new directory or project
     * - change mode (curves or shade)
     * 
     * TabbedPane must be updated
     */
    private void setVisibleSample(int sampleIndex) {
        _setVisibleSample(sampleIndex);
        scaleAndRepaint();
        updateButtons();

        String string = "setVisibleSample"+";"+ String.valueOf(sampleIndex);
        writeInSaveFile (string);

    }
    private void _setVisibleSample(int sampleIndex) {

        /*
         * changement eventuel du ladder selectionné
         */
        if (selectedSampleIndex != sampleIndex) {
            loadingProject=true;
            selectedSampleIndex = sampleIndex;
            jTabbedPane1.setSelectedIndex(selectedSampleIndex);
            loadingProject = false;
        }

        selectedSample = tab_samples.getObject(selectedSampleIndex);
        if (selectedSample == null) return;
        selectedGuiLadder = tab_guiLadders.getObject(selectedSampleIndex);
        
        /*
         * calcul du viewAlignMode qui va etre utilise
         * suivant ce qui est possible
         * suivant ce qui est demande
         */
        
        /*
         * mise a jour eventuelle du ladder
         */
        int sampleViewAlignMode = selectedSample.getBestViewFor(viewAlignMode);
         if (sampleViewAlignMode != selectedGuiLadder.getAlignMode()) {
            selectedSample.viewMode = sampleViewAlignMode;
            selectedGuiLadder.viewAlignedDatas(sampleViewAlignMode);
        }

        /*
         * mise a jour eventuelle des datas
         */
        jPanel1.removeAll();

        guiSample.setSample(selectedSample,sampleViewAlignMode);
        shadeSample.setSample(selectedSample,sampleViewAlignMode);
        if (sampleViewMode==CURVES_MODE) {
            jPanel1.add(guiSample, java.awt.BorderLayout.CENTER);
        }
        else if (sampleViewMode == SHADE_MODE) {
            jPanel1.add(shadeSample, java.awt.BorderLayout.CENTER);
        }
        jPanel1.validate();
    }
    


    /*
    public boolean isLocked() {
        return lockButton.isSelected();
    }
     * 
     */
    
    public void export() {
        
        File exportDir = new File(projectFile,"aligned_datas");
        
        exportDir.mkdirs();

        LadderFileChooser fileChooser = new LadderFileChooser(null,true,exportDir);
        fileChooser.allowOnlyDirectory();
        File file = fileChooser.getFile ("Exporter");
        if (file == null) return;
        
        exportDir = file;


        /*
         * demande de confirmation d'ecrasement eventuelle
         */
        String[] files = exportDir.list();
        if (files != null && files.length>0) {
            int n = JOptionPane.showConfirmDialog(this,"Directory "+exportDir+ " not empty; Cleaning before writing ?", "", JOptionPane.YES_NO_OPTION);
            if (n == JOptionPane.NO_OPTION) {
                return;
            }
            FileUtil.makeDirClean(exportDir);
        }
        
        /*
         * chox de la zone a exporter
         */
        int minima = (int)(refSample.xMinOfDatasets);
        int maxima = (int)(refSample.xMaxOfDatasets);
        boolean allRange = false;
        try {
            TuiExport tuiExport = new TuiExport(null,"Selection of range","",
            "Normalize on all range", allRange,
            "minimum",new Integer(minima),"maxima",new Integer(maxima),
            new Integer(minima),new Integer(maxima));
            minima = tuiExport.getX();
            maxima = tuiExport.getY();
            allRange = tuiExport.getB1();
        } catch (TuiException ex) {
            return;
        }
        
        /*
         * exportation
         */
        writeMsg("> Export aligned datas in directory " + exportDir.getPath()+'\n');
        for (int i=0;i<tab_samples.size();i++) {
            Sample sample = tab_samples.getObject(i);
            if (sample.isLadderAligned()) {
                try {
                    sample.export(exportDir,(double)minima,(double)maxima,allRange,1);
                    writeMsg(">     Export " + sample.repName+'\n');
                }
                catch (IOException ex) {
                    writeMsg("> Error : no writing  "+'\n');
                } 
            }
        }
        scaleAndRepaint();
    }
    
    private int lookForGuiDataSet (String name) {
        return tab_guiLadders.getIndexFromKey(name);
    }
    private GuiLadder getGuiDataSet (int index) {
        return tab_guiLadders.getObject(index);
    }
    public void writeMsg(String msg) {
        outPanel.write(msg);
    }
    public void writeError(String msg) {
        outPanel.write(msg,Color.RED,0);
    }
    public void printLadder() {
        GuiLadder guiLadder = ((GuiLadder)(jTabbedPane1.getSelectedComponent()));
        if (guiLadder != null) guiLadder.printLadder();
    }
    public void printSample() {
        if (sampleViewMode==CURVES_MODE) {
            guiSample.printSample();
        }
        else if (sampleViewMode == SHADE_MODE) {
            shadeSample.printSample();
        }
    }


/** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane0 = new javax.swing.JSplitPane();
        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel2 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();

        setLayout(new java.awt.BorderLayout());

        jSplitPane0.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane0.setResizeWeight(1.0);
        jSplitPane0.setOneTouchExpandable(true);

        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.setResizeWeight(0.5);
        jSplitPane1.setAutoscrolls(true);
        jSplitPane1.setOneTouchExpandable(true);
        jSplitPane1.setVerifyInputWhenFocusTarget(false);

        jPanel2.setLayout(new java.awt.BorderLayout());
        jSplitPane1.setTopComponent(jPanel2);

        jPanel1.setLayout(new java.awt.BorderLayout());
        jSplitPane1.setBottomComponent(jPanel1);

        jSplitPane0.setTopComponent(jSplitPane1);

        add(jSplitPane0, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JSplitPane jSplitPane0;
    private javax.swing.JSplitPane jSplitPane1;
    // End of variables declaration//GEN-END:variables
    
}
