/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * 
 * Copyright 2015 Riccardo Scorretti, Olivier Fabregues, Sandrine Demaneche, Elisabeth Navarro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */
package risaaligner.datas;

import java.awt.Color;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;
import org.apache.commons.math3.analysis.interpolation.SplineInterpolator;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;

/**
 *
 * @author olivier
 */
public class DataSet {
    String localFileName = "";
    String appliName = "";
    String fileName = "";
    String shortFileName = "";
    String filePath = "";
    String dateCreated = "";
    String dateModified = "";
    String versionCreated = "";
    String versionModified = "";
    String assayName = "";
    String assayPath = "";
    String assayTitle = "";
    String asayVersion = "";
    int numberOfSamplesRun = 0;
    public String sampleName = "";
    int numberOfEvents = 0;
    
    public double[] xValues = null;
    public double[] yValues = null;
    public double xMin = 0.;
    public double xMax = 0.;
    public double yMin = 0.;
    public double yMax = 0.;
    
    /* pour debug, a repasser en variable locale
     * 
     */
    
    public double[] yAlignValues = null;
    public double yMinAlign = 0.;
    public double yMaxAlign = 0.;
    
    public double[] yAlign2Values = null;
    public double yMinAlign2 = 0.;
    public double yMaxAlign2 = 0.;
    
    public SpotLine[] spotLines;
    /*
    public int[] spotLinesIndexes;
    public int[] spotLinesIndexesAlign;
    public int[] spotLinesStates;
     * 
     */
    
    public Color color = Color.black;
    
    NumberFormat xFormat; 
    NumberFormat yFormat; 
    NumberFormat yScFormat; 

    String newLine = new String(new char[]{'\n'});

    int range = 20;

    PolynomialSplineFunction psfAlign = null;

    
    public DataSet(String filePath) throws Exception {
        this (new File(filePath));
    }
    public DataSet(File file) throws Exception {
        read(file);
    }
    public void read(File file) throws Exception {
        
        xFormat = NumberFormat.getInstance(Locale.US);
        if (xFormat instanceof DecimalFormat) {
            ((DecimalFormat) xFormat).setDecimalSeparatorAlwaysShown(true);
            ((DecimalFormat) xFormat).setMaximumFractionDigits(2);
        }
        yFormat = NumberFormat.getInstance(Locale.US);
        if (yFormat instanceof DecimalFormat) {
            ((DecimalFormat) yFormat).setDecimalSeparatorAlwaysShown(true);
            ((DecimalFormat) yFormat).setMaximumFractionDigits(7);
        }
        yScFormat = NumberFormat.getInstance(Locale.US);
        if (yScFormat instanceof DecimalFormat) {
            ((DecimalFormat) yScFormat).applyPattern("0.######E00");
        }
 
        

        localFileName = file.getName();
        int index = file.getName().lastIndexOf('_');
        if (index >=0) appliName = file.getName().substring(index+1);
        BufferedReader br = null;
        br = new BufferedReader(new FileReader(file));

        String line = null;

        while ((line = br.readLine()) != null) {

            if (line != null && line.length()!=0) {

                String[] fields = line.split(",");

                // debut de la lecture des données
                if (fields[0].equals("Data File Name")) {
                    fileName = fields[1];
                }
                else if (fields[0].equals("Data File Path")) {
                    filePath = fields[1];
                }
                else if (fields[0].equals("Date Created")) {
                    dateCreated = fields[1];
                }
                else if (fields[0].equals("Date Last Modified")) {
                    dateModified = fields[1];
                }

                else if (fields[0].equals("Version Created")) {
                    versionCreated = fields[1];
                }
                else if (fields[0].equals("Version Last Modified")) {
                    versionModified = fields[1];
                }
                else if (fields[0].equals("Assay Name")) {
                    assayName = fields[1];
                }
                else if (fields[0].equals("Assay Path")) {
                    assayPath = fields[1];
                }
                else if (fields[0].equals("Assay Title")) {
                    assayTitle = fields[1];
                }
                else if (fields[0].equals("Assay Version")) {
                    asayVersion = fields[1];
                }
                else if (fields[0].equals("Number of Samples Run")) {
                    numberOfSamplesRun = Integer.valueOf(fields[1]).intValue();
                }
                else if (fields[0].equals("Sample Name")) {
                    sampleName = fields[1];
                }
                else if (fields[0].equals("Number of Events")) {
                    numberOfEvents = Integer.valueOf(fields[1]).intValue();
                }
                else if (fields[0].equals("Time")) {
                    if (fields[1].equals ("Value")) break;
                }
            }
        }

        if (numberOfEvents <= 0) throw new Exception();

        xValues = new double[numberOfEvents];
        yValues = new double[numberOfEvents];

        for (int i=0;i<numberOfEvents;i++) {
        //while ((line = br.readLine()) != null) {


            line = br.readLine();

            if (line != null && line.length()!=0) {

                String[] fields = line.split(",");

                double x = Double.valueOf(fields[0]).doubleValue();
                double y = Double.valueOf(fields[1]).doubleValue();

                if (y <-3.) {
                    int truc = 2;
                }
                xValues[i] = x;
                yValues[i] = y;

                if ( i==0 || x<xMin) xMin = x;
                if ( i==0 || x>xMax) xMax = x;
                if ( i==0 || y<yMin) yMin = y;
                if ( i==0 || y>yMax) yMax = y;
            }
        }

            
        spotLines = new SpotLine[0];
        /*
        spotLinesIndexes = new int[0];
        spotLinesStates = new int[0];
        spotLinesIndexesAlign = new int[0];
         * 
         */
    }

    public void createSpotLine(double xValue,  int state, boolean visible) {
        
        int xIndex = getBestXIndexFromValue(xValue);
        if (xIndex <0) return;
        int index = getNewLineSpotIndexForX(xIndex);
        //state = SpotLine.NORMAL;
        //visible = true;
        SpotLine[] newSpotLines = new SpotLine[spotLines.length+1];
        System.arraycopy(spotLines, 0, newSpotLines, 0, index);
        newSpotLines[index] = new SpotLine(index,xIndex,state,visible);
        System.arraycopy(spotLines, index, newSpotLines, index+1, spotLines.length-index);
        spotLines = newSpotLines;
        updateIndex();
    }
    public void createSpotLine(int index,int xIndex,int state, boolean visible) {
        
        SpotLine[] newSpotLines = new SpotLine[spotLines.length+1];
        System.arraycopy(spotLines, 0, newSpotLines, 0, index);
        newSpotLines[index] = new SpotLine(index,xIndex,state,visible);
        System.arraycopy(spotLines, index, newSpotLines, index+1, spotLines.length-index);
        spotLines = newSpotLines;
        updateIndex();
    }

    public void removeSpotLine(int index) {
        if (index >=0) {
            SpotLine[] newSpotLines = new SpotLine[spotLines.length-1];
            System.arraycopy(spotLines, 0, newSpotLines, 0, index);
            System.arraycopy(spotLines, index+1, newSpotLines, index, spotLines.length-index-1);
            spotLines = newSpotLines;
            updateIndex();
        }
    }
    public int changeSpotLineState(int index) {
        return spotLines[index].changeState();
    }

    public void updateIndex() {
        for (int i=0;i<spotLines.length;i++) spotLines[i].index = i;
    }



    public void moveSpotLine(int index,double xValue) {
        int xIndex = -1;
        if (spotLines[index].state == SpotLine.NORMAL){
            xIndex = getBestXIndexFromValue(xValue);
        }
        else {
            xIndex = getXIndexFromValue(xValue);
        }
        if (xIndex >= 0) {
            if (index >0) {
                if (xIndex <= spotLines[index-1].xIndex0) xIndex = spotLines[index-1].xIndex0+1;
            }
            if (index < spotLines.length-1) {
                if (xIndex >= spotLines[index+1].xIndex0) xIndex = spotLines[index+1].xIndex0-1;
            }
            spotLines[index].xIndex0 = xIndex;
        }
    }
    
    public int getXIndexFromValue(double value) {
        int noValue = -1;
        if (value < xValues[0]) return 0;
        if (value > xValues[xValues.length-1])return xValues.length-2;
        for (int i=1;i<xValues.length;i++) {
            if (xValues[i]>value) {
                return (i-1);
            }
        }
        return noValue;
    }
    public int getBestXIndexFromValue(double value) {
        int noValue = getXIndexFromValue(value);
        
        int noValMin = noValue - range;
        int noValMax = noValue + range;
        if (noValMin <=0) noValMin = 0;
        if (noValMax >= xValues.length-2) noValMax = xValues.length-2;

        int noBestValue = noValue; 
        double maxValue = yAlignValues[noValue];
        int distance = xValues.length;
        double oldSens = 0.;
        double oldXValue = 0.;
        for (int i=noValMin;i<=noValMax;i++) {
            double sens = yAlignValues[i]-oldXValue;
            double delta = sens*oldSens;
            int localDistance = Math.abs(i-noValue);
            if (i>=noValMin+2 && sens <0.&& delta<0. && localDistance < distance && yAlignValues[i] > maxValue) {
                noBestValue = i;
                distance = localDistance;
                maxValue = yAlignValues[i];
            }
            oldXValue = yAlignValues[i];
            oldSens = sens;
        }
        return noBestValue;
    }
    public int getNewLineSpotIndexForX (int xIndex) {
        int index = spotLines.length;
        for (int i=0;i<spotLines.length;i++) {
            if (spotLines[i].xIndex>xIndex) {
                index = i;
                break;
            }
        }
        return index;
    }

    
    
    
    
    
    
    public void removeAlign () {
        yAlignValues = null;
    }
    
    public void alignCurve (PolynomialSplineFunction psf) throws Exception  {
        
        /*
         * si c est la reference, on recopie
         */
        if (psf == null) {
            yAlignValues = new double[numberOfEvents];
            for (int i=0;i<numberOfEvents;i++) {
                if (yValues[i] <0.) {
                    int truc = 2;
                }
                yAlignValues[i] = yValues[i]; 
            }
            yMinAlign = yMin;
            yMaxAlign = yMax;
        }
        
        /*
         * on interpole pour recaler les sommets
         */
        else {
            

            /*
             * premiere interpolation pour recaler les abcisses
             */
            double[] _xLocalValues = new double[xValues.length];
            for (int i=0;i<xValues.length;i++) {
                _xLocalValues[i] = psf.value(xValues[i]);
                if ((i>0) && (_xLocalValues[i]<=_xLocalValues[i-1])) _xLocalValues[i] = _xLocalValues[i-1]+1.; 
            }

            /*
             * deuxieme interpolation pour trouver les ordonnées aux abcisses d'origine
             */
            SplineInterpolator interpolator = new SplineInterpolator();
            PolynomialSplineFunction  psf2 = interpolator.interpolate(_xLocalValues, yValues);

            yAlignValues = new double[xValues.length];
            for (int i=0;i<xValues.length;i++) {
                double y =  psf2.value(xValues[i]);
                //if (y<0.) y=0.;
                yAlignValues[i] = y; 
                if ( i==0 || y<yMinAlign) yMinAlign = y;
                if ( i==0 || y>yMaxAlign) yMaxAlign = y;
            }
        }
    }
    public void removeAlignDatas () {
        yAlign2Values = null;
    }
    public void alignDatas (PolynomialSplineFunction psf)  throws Exception {
        psfAlign = psf;
        /*
         * si c est la reference, on recopie
         */
        if (psf == null) {
            for (int i=0;i<spotLines.length;i++) {
                spotLines[i].xIndex1 = spotLines[i].xIndex0;
            }
            yAlign2Values = new double[numberOfEvents];
            for (int i=0;i<numberOfEvents;i++) {
                yAlign2Values[i] = yAlignValues[i]; 
            }
            yMinAlign2 = yMinAlign;
            yMaxAlign2 = yMaxAlign;
        }
        
        /*
         * on interpole pour recaler les sommets
         */
        else {

            /*
             * recalage des spots
             */
            for (int i=0;i<spotLines.length;i++) {
                double oldXSpot = xValues[spotLines[i].xIndex0];
                double newYSpot = psf.value(oldXSpot);
                spotLines[i].xIndex1 = getXIndexFromValue(xValues,newYSpot);
            }
            /*
             * premiere interpolation pour recaler les abcisses
             */
            double[] xLocalValues = new double[numberOfEvents];
            for (int i=0;i<numberOfEvents;i++) {
                xLocalValues[i] = psf.value(xValues[i]);
                if ((i>0) && (xLocalValues[i]<xLocalValues[i-1])) xLocalValues[i] = xLocalValues[i-1]+1.; 
            }

            /*
             * deuxieme interpolation pour trouver les ordonnées aux abcisses d'origine
             */
            SplineInterpolator interpolator = new SplineInterpolator();
            PolynomialSplineFunction  psf2 = interpolator.interpolate(xLocalValues, yAlignValues);

            yAlign2Values = new double[xValues.length];
            for (int i=0;i<xValues.length;i++) {
                double y =  psf2.value(xValues[i]);
                //if (y<0.) y=0.;
                yAlign2Values[i] = y; 
                if ( i==0 || y<yMinAlign2) yMinAlign2 = y;
                if ( i==0 || y>yMaxAlign2) yMaxAlign2 = y;
            }
        }
    }
    public double getSpotValue(int noSpot) {
        return (xValues[spotLines[noSpot].xIndex]+xValues[spotLines[noSpot].xIndex+1])/2.;
    }
    public double getMinValue() {
        return xValues[0];
    }
    public double getMaxValue() {
        return xValues[yValues.length-1];
    }

    public int getXIndexFromValue(double[] values,double value) {
        int noValue = -1;
        if (value >= values[0] && value <= values[values.length-1]) {
            for (int i=1;i<values.length;i++) {
                if (values[i]>value) {
                    noValue = i-1;
                    break;
                }
            }
        }
        return noValue;
    }
    public double getSpotPosition( int noSpot) {
        return (xValues[spotLines[noSpot].xIndex0]+xValues[spotLines[noSpot].xIndex0+1])/2.;
    }

       
   String getYValue(double value) {
        String sValue;
        if (value == 0.) {
            sValue = "0.";
        }
        else if (Math.abs(value)>=0.1) {
            sValue = yFormat.format(value);
        }
        else {
            sValue = yScFormat.format(value);
        }
        return sValue;
    }
    public void write(File rep,double minima, double maxima,boolean allRange,int alignedType,boolean negativ) throws IOException {

        String baseName = new String(localFileName);
        String extension = ".csv";
        int index = localFileName.lastIndexOf('.');
        if (index >=0) {
            baseName = localFileName.substring(0,index);
            extension = localFileName.substring(index,localFileName.length());
        }
        String exportName = baseName.concat("_aligned");
        if (alignedType == 2) {
            exportName = baseName.concat("_aligned2");
        }
        if (negativ) {
            exportName = exportName.concat("_negat");
        }
        exportName = exportName.concat(extension);
        
        
        File file = new File (rep,exportName);
        FileWriter writer = new FileWriter(file);
        BufferedWriter w = new BufferedWriter(writer);

        double[] _xValues = xValues;
        double[] _yValues = yAlignValues;
        if (alignedType == 2 ) {
            _yValues = yAlign2Values;
        }
        
        int nbValues = 0;
        double[] xLocalValues = new double[numberOfEvents];
        double[] yLocalValues = new double[numberOfEvents];

        double yLocalMax = 0;
        if (!negativ) {
            for (int i=0;i<_xValues.length;i++) {
                double x = _xValues[i];
                double y = _yValues[i];
                if (y<0) y = 0;
                if (x>=minima && x<=maxima) {
                    xLocalValues[nbValues] = x;
                    yLocalValues[nbValues] = y;
                    yLocalMax += y;
                    nbValues++;
                }
                else if (allRange) {
                    yLocalMax += y;
                }
            }
        }
        else {
            double yMin = 0.;
            for (int i=0;i<_xValues.length;i++) {
                double x = _xValues[i];
                double y = _yValues[i];
                if (x>=minima && x<=maxima) {
                    if (nbValues==0 || y<yMin) yMin = y;
                    xLocalValues[nbValues] = x;
                    yLocalValues[nbValues] = y;
                    yLocalMax += y;
                    nbValues++;
                }
                else if (allRange) {
                    yLocalMax += y;
                }
            }
            for (int i=0;i<nbValues;i++) {
                yLocalValues[i]-=yMin;
            }
            if (allRange) {
                yLocalMax -= (_xValues.length * yMin);
            }
            else {
                yLocalMax -= (nbValues * yMin);
            }
            
        }
        
        
        w.write ("Data File Name,"+fileName+newLine);
        w.write ("Data File Path,"+filePath+newLine);
        w.write ("Date Created,"+dateCreated+newLine);
        w.write ("Date Last Modified,"+dateModified+newLine);
        w.write ("Version Created,"+versionCreated+newLine);
        w.write ("Version Last Modified,"+versionModified+newLine);
        w.write (newLine);
        
        w.write ("Assay Name,"+assayName+newLine);
        w.write ("Assay Path,"+assayPath+newLine);
        w.write ("Assay Title,"+assayTitle+newLine);
        w.write ("Assay Version,"+asayVersion+newLine);
        w.write (newLine);
        
        w.write ("Number of Samples Run,"+numberOfSamplesRun+newLine);
        w.write (newLine);
        
        w.write ("Sample Name,"+sampleName+newLine);
        w.write ("Number of Events,"+nbValues+newLine);
        w.write (newLine);

        w.write ("Time,Value"+newLine);
        
        
            
        for (int i=0;i<nbValues;i++) {
            double y = yLocalValues[i]/yLocalMax*100.;
            if (y<0.) y=0.;
            if (y>100.) y=100.;
            w.write (xFormat.format(xLocalValues[i])+","+getYValue(y)+newLine);
        }

            
        w.write ("Alignment,On"+newLine);
        w.close();
    }
}
