/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * 
 * Copyright 2015 Riccardo Scorretti, Olivier Fabregues, Sandrine Demaneche, Elisabeth Navarro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */
package risaaligner.datas;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;
import java.util.Vector;
import org.apache.commons.math3.analysis.interpolation.SplineInterpolator;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;
import risaaligner.RisaAligner;
import risaaligner.TabPanel;

/**
 *
 * @author olivier
 */
public class Sample {
    public String repName;
    public String repPath;
    public Sample refSample;
    public DataSet ladderDataSet;
    public DataSet[] dataSets;

    public TabPanel tabPanel;
    public double xMinOfDatasets = 0.;
    public double xMaxOfDatasets = 0.;
    public double yMinOfDatasets = 0.;
    public double yMaxOfDatasets = 0.;
    
    public double xMinTotal = 0.;
    public double xMaxTotal = 0.;
    public double yMinTotal = 0.;
    public double yMaxTotal = 0.;

    public Spot[] spots ;
    public Spot[] refSpots ;
    public double level = 0.;

   
    public PolynomialSplineFunction psf = null;

    boolean alignable = false;
    boolean ladderAligned = false;
    boolean datasAligned = false;

    NumberFormat xFormat; 
    NumberFormat yFormat; 
    NumberFormat yScFormat; 

    String newLine = new String(new char[]{'\n'});
    
    public int bestViewMode;
    public int viewMode;

    public Sample (String repName,String repPath,Sample refSample,DataSet ladderDataSet,DataSet[] dataSets) {
        this.repName = repName;
        this.repPath = repPath;
        this.refSample = refSample;
        this.ladderDataSet = ladderDataSet;
        this.dataSets = dataSets;
        
        boolean first = true;
        for (int i=0;i<dataSets.length;i++) {
            DataSet dataSet = dataSets[i];
            if (first || dataSet.xMin<xMinOfDatasets) xMinOfDatasets = dataSet.xMin;
            if (first || dataSet.xMax>xMaxOfDatasets) xMaxOfDatasets = dataSet.xMax;
            if (first || dataSet.yMin<yMinOfDatasets) yMinOfDatasets = dataSet.yMin;
            if (first || dataSet.yMax>yMaxOfDatasets) yMaxOfDatasets = dataSet.yMax;
            first = false;
        }
        xMinTotal = ladderDataSet.xMin;
        xMaxTotal = ladderDataSet.xMax;
        yMinTotal = ladderDataSet.yMin;
        yMaxTotal = ladderDataSet.yMax;
        if (xMinOfDatasets<xMinTotal) xMinTotal = xMinOfDatasets;
        if (yMinOfDatasets<yMinTotal) yMinTotal = yMinOfDatasets;
        if (xMaxOfDatasets>xMaxTotal) xMaxTotal = xMaxOfDatasets;
        if (yMaxOfDatasets>yMaxTotal) yMaxTotal = yMaxOfDatasets;

        /*
         * creation d'un niveau moyen
         */
        level = ladderDataSet.yMin + (ladderDataSet.yMax-ladderDataSet.yMin)*0.25;

        /*
         * creation des spots au points maximum
         */
        Vector<Spot> vSpots = new Vector<Spot>();
        double oldXValue = 0.;
        double oldYValue = 0.;
        double oldSens = 0.;
        for (int i=0;i<ladderDataSet.numberOfEvents;i++) {
            double sens = ladderDataSet.yValues[i]-oldYValue;
            double delta = sens*oldSens;
            if (i>=2 && delta<0. && sens<0.) {
                vSpots.addElement(new Spot(vSpots.size(),Spot.LADDER,oldXValue,oldYValue));
            }
            oldXValue = ladderDataSet.xValues[i];
            oldYValue = ladderDataSet.yValues[i];
            oldSens = sens;
        }
        spots = new Spot[vSpots.size()];
        vSpots.copyInto(spots);
    }
    public void setLevel(double level) {
        this.level = level;
    }
    public int changeSpotState (int spotType,int spotIndex) {
        Spot spot = null;
        if (spotType == Spot.LADDER) {
            spot = spots[spotIndex];
        }
        else if (spotType == Spot.LADDER_REF) {
            spot = refSpots[spotIndex];
        }
        return spot.changeState();
    }
    public void setSpotState (int spotType,int spotIndex,int state) {
        Spot spot = null;
        if (spotType == Spot.LADDER) {
            spot = spots[spotIndex];
        }
        else if (spotType == Spot.LADDER_REF) {
            spot = refSpots[spotIndex];
        }
        spot.setState(state);
    }
    public boolean isReference() {
        if (this == refSample) return true;
        return false;
    }
    
    public boolean hasReference() {
        if (refSample == null) return false;
        return true;
    }
    
    public boolean isAlignable() {
        return alignable;
    }
    public boolean isLadderAligned() {
        return ladderAligned;
    }
    public boolean isDatasAligned() {
        return datasAligned;
    }

    public int getBestViewFor (int viewMode) {
        this.bestViewMode = viewMode;
        if (viewMode == RisaAligner.VIEW_DATAS_ALIGNED){
            if (datasAligned) {
                bestViewMode = RisaAligner.VIEW_DATAS_ALIGNED;
            }
            else if (ladderAligned) {
                bestViewMode = RisaAligner.VIEW_LADDER_ALIGNED;
            }
            else {
                bestViewMode = RisaAligner.VIEW_SAMPLE;
            }
        }
        else if (viewMode == RisaAligner.VIEW_LADDER_ALIGNED){
            if (datasAligned || ladderAligned) {
                bestViewMode = RisaAligner.VIEW_LADDER_ALIGNED;
            }
            else {
                bestViewMode = RisaAligner.VIEW_SAMPLE;
            }
        }
        else {
            bestViewMode = RisaAligner.VIEW_SAMPLE;
        }
        return bestViewMode;
    }
    
    public void setRefSample (Sample refSample){
        if (isLadderAligned()) removeAlignLadder();

        this.refSample = refSample;
        refSpots = new Spot[refSample.spots.length];
        for (int i=0;i<refSample.spots.length;i++) {
            refSpots[i] = new Spot(refSample.spots[i],Spot.LADDER_REF);
        }
    }
    public void removeRefSample (){
        if (isLadderAligned()) removeAlignLadder();

        this.refSample = null;
        refSpots = null;
    }
    public void setAlignable(boolean alignable) {
        this.alignable = alignable;
        tabPanel.setAlignable(alignable);
    }

    public void alignLadder(PolynomialSplineFunction psf) throws Exception {
        this.psf = psf;
        if (isReference()) {
            psf = null;
        }
        ladderDataSet.alignCurve (psf);
        for (int i=0;i<dataSets.length;i++) {
            dataSets[i].alignCurve(psf);
        }
        if (psf!=null) {
            
            /*
             * recalage des spots
             */
            for (int i=0;i<spots.length;i++) {
                Spot spot = spots[i];
                spot.alignX = psf.value(spot.x);
            }
        }

        ladderAligned = true;
        tabPanel.setLadderAligned(true);
    }

    public void removeAlignLadder () {
        if (datasAligned) removeAlignDatas();
        psf = null;
        if (ladderAligned) {
            ladderDataSet.removeAlign();
            for (int i=0;i<dataSets.length;i++) {
                dataSets[i].removeAlign();
            }
            for (int i=0;i<spots.length;i++) {
                Spot spot = spots[i];
                spot.alignX = 0.;
            }
            tabPanel.setLadderAligned(false);
        }
        ladderAligned = false;
    }
    public void createSpotLineAuto(double xValue,int state, boolean visible) {
        ladderDataSet.createSpotLine(xValue,SpotLine.FREE,visible);
        for (int i=0;i<dataSets.length;i++) {
            dataSets[i].createSpotLine(xValue,state,visible);
        }
    }
    public void createSpotLine(int dataSetIndex,int spotLineIndex,int xIndex, int state, boolean visible) {
        if (dataSetIndex <0) {
            ladderDataSet.createSpotLine(spotLineIndex,xIndex,state,visible);
        }
        else {
            dataSets[dataSetIndex].createSpotLine(spotLineIndex,xIndex,state,visible);
        }
    }
    public void removeSpotLine(int index) {
        ladderDataSet.removeSpotLine(index);
        for (int i=0;i<dataSets.length;i++) {
            dataSets[i].removeSpotLine(index);
        }
    }
    public void changeSpotLineState(int noDataSet,int index) {
        if (noDataSet <0) {
            ladderDataSet.changeSpotLineState(index);
        }
        else {
            dataSets[noDataSet].changeSpotLineState(index);
        }
    }
    public void moveSpotLine(int noDataSet,int index,double xValue) {
        if (noDataSet <0) {
            ladderDataSet.moveSpotLine(index,xValue);
            for (int i=0;i<dataSets.length;i++) {
                dataSets[i].moveSpotLine(index,xValue);
            }
        }
        else {
            dataSets[noDataSet].moveSpotLine(index,xValue);
        }
    }
    
    public PolynomialSplineFunction[] getSplineFunction () {
        PolynomialSplineFunction[] psfs = new PolynomialSplineFunction[dataSets.length+1];

        for (int i=0;i<dataSets.length+1;i++) {
            
            SpotLine[] refSpotLines = ladderDataSet.spotLines;
            /*
             * recup du polynome
             */
            PolynomialSplineFunction psf = null;
            if (i>0) {
                DataSet dataSet = dataSets[i-1];
                SpotLine[] localSpotLines = dataSet.spotLines;
                
                
                int nbSpots = 0;
                for (int j=0;j<refSpotLines.length;j++) {
                    if (refSpotLines[j].state<=1 && localSpotLines[j].state<=1) nbSpots++;
                }
                double[] xLocal = new double[nbSpots+2];
                double[] yLocal = new double[nbSpots+2];
                xLocal[0] = dataSet.getMinValue();
                yLocal[0] = ladderDataSet.getMinValue();
                nbSpots = 1;
                for (int j=0;j<refSpotLines.length;j++) {
                    if (refSpotLines[j].state<=1 && localSpotLines[j].state<=1) {
                        xLocal[nbSpots] = dataSet.getSpotPosition(j);
                        yLocal[nbSpots] = ladderDataSet.getSpotPosition(j);
                        nbSpots++;
                    }
                }
                xLocal[nbSpots] = dataSet.getMaxValue();
                yLocal[nbSpots] = ladderDataSet.getMaxValue();
                nbSpots++;
                SplineInterpolator interpolator = new SplineInterpolator();
                if (nbSpots>=3) psf = interpolator.interpolate(xLocal, yLocal);
            }
            psfs[i] = psf;
        }
        return psfs;
        
        //sample.alignDatas(psfs);
    }
    public void alignDatas () throws Exception {
        PolynomialSplineFunction[] psfs = getSplineFunction ();
        if (ladderAligned) {
            ladderDataSet.alignDatas(psfs[0]);
            for (int i=0;i<dataSets.length;i++) {
                dataSets[i].alignDatas(psfs[i+1]);
            }
            datasAligned = true;
            tabPanel.setDatasAligned(true);
        }
    }
    public void removeAlignDatas () {
        if (datasAligned) {
            ladderDataSet.removeAlignDatas();
            for (int i=0;i<dataSets.length;i++) {
                dataSets[i].removeAlignDatas();
            }
            datasAligned = false;
            tabPanel.setDatasAligned(false);
        }
    }

    /*
     * export
     */
    public void export (File rep,double minima,double maxima,boolean allRange,int alignedType) throws IOException {
        if ( ladderAligned ) {
            File outRep = new File(rep,repName);
            outRep.mkdirs();

            /*
             * Ecriture du ladder dans un fichier separe 
             */
            if (ladderAligned) {
                ladderDataSet.write(outRep,minima,maxima,allRange,1,true);
                ladderDataSet.write(outRep,minima,maxima,allRange,1,false);
            }
            if (datasAligned) {
                ladderDataSet.write(outRep,minima,maxima,allRange,2,true);
                ladderDataSet.write(outRep,minima,maxima,allRange,2,false);
            }
            /*
             * ecriture des courbes sans un fichier separe
             */
            for (int i=0;i<dataSets.length;i++) {
                if (ladderAligned) {
                     dataSets[i].write(outRep,minima,maxima,allRange,1,true);
                     dataSets[i].write(outRep,minima,maxima,allRange,1,false);
                }
                if (datasAligned) {
                     dataSets[i].write(outRep,minima,maxima,allRange,2,true);
                     dataSets[i].write(outRep,minima,maxima,allRange,2,false);
                }
            }
            
            /*
             * ecriture de toutes les donnees dans le meme fichier
             */
            if (ladderAligned) {
                write(outRep,minima,maxima,allRange,1,true);
                write(outRep,minima,maxima,allRange,1,false);
            }
            if (datasAligned) {
                write(outRep,minima,maxima,allRange,2,true);
                write(outRep,minima,maxima,allRange,2,false);
            }
        }
    }
    
    void write (File rep,double minima, double maxima,boolean allRange,int alignedType,boolean negativ) throws IOException {
        xFormat = NumberFormat.getInstance(Locale.US);
        if (xFormat instanceof DecimalFormat) {
            ((DecimalFormat) xFormat).setDecimalSeparatorAlwaysShown(true);
            ((DecimalFormat) xFormat).setMaximumFractionDigits(2);
        }
        yFormat = NumberFormat.getInstance(Locale.US);
        if (yFormat instanceof DecimalFormat) {
            ((DecimalFormat) yFormat).setDecimalSeparatorAlwaysShown(true);
            ((DecimalFormat) yFormat).setMaximumFractionDigits(7);
        }
        yScFormat = NumberFormat.getInstance(Locale.US);
        if (yScFormat instanceof DecimalFormat) {
            ((DecimalFormat) yScFormat).applyPattern("0.######E00");
        }

        String baseName = new String(repName);
        String extension = ".csv";
        String exportName = baseName.concat("_aligned");
        if (alignedType == 2) {
            exportName = baseName.concat("_aligned2");
        }
        if (negativ) {
            exportName = exportName.concat("_negat");
        }
        exportName = exportName.concat(extension);

        File file = new File (rep,exportName);
        FileWriter writer = new FileWriter(file);
        BufferedWriter w = new BufferedWriter(writer);

        String[] names = new String[dataSets.length+1];
        

        /*
         * balayage du ladder pour remplir le tableau des abcisses 
         */
        double[] _xValues = ladderDataSet.xValues;
        double[] _yValues = ladderDataSet.yAlignValues;
        double[] xLocalValues = new double[_xValues.length];
        double[][] yLocalValues = new double[dataSets.length+1][];
        
        int nbValues = 0;
        for (int i=0;i<_xValues.length;i++) {
            double x = _xValues[i];
            if (x>=minima && x<=maxima) {
                xLocalValues[nbValues] = x;
                nbValues++;
            }
        }
        
        /*
         * balayage des datasets pour recuperer les noms et les ordonnées et le cumul
         */
        double[] yLocalMax = new double[dataSets.length+1];
        
        for (int i=0;i<dataSets.length+1;i++) {
            String name;
            if (i==0 ) {
                _xValues = ladderDataSet.xValues;
                _yValues = ladderDataSet.yAlignValues;
                if (alignedType == 2) {
                    _yValues = ladderDataSet.yAlign2Values;
                }
                name = "Ladder";
            }
            else {
                _xValues = dataSets[i-1].xValues;
                _yValues = dataSets[i-1].yAlignValues;
                if (alignedType == 2) {
                    _yValues = dataSets[i-1].yAlign2Values;
                }
                name = dataSets[i-1].sampleName;
            }
            names[i] = name;
            yLocalValues[i] = new double[_xValues.length];

            
            nbValues = 0;
            yLocalMax[i] = 0;
            if (!negativ) {
                for (int j=0;j<_xValues.length;j++) {
                    double x = _xValues[j];
                    double y = _yValues[j];
                    if (y<0) y = 0;
                    if (x>=minima && x<=maxima) {
                        yLocalValues[i][nbValues] = y;
                        yLocalMax[i] += y;
                        nbValues++;
                    }
                    else if (allRange) {
                        yLocalMax[i] += y;
                    }
                }
            }
            else {
                double yMin = 0;
                for (int j=0;j<_xValues.length;j++) {
                    double x = _xValues[j];
                    double y = _yValues[j];
                    if (x>=minima && x<=maxima) {
                        if (nbValues==0 || y<yMin) yMin = y;
                        yLocalValues[i][nbValues] = y;
                        yLocalMax[i] += y;
                        nbValues++;
                    }
                    else if (allRange) {
                        yLocalMax[i] += y;
                    }
                }
                for (int j=0;j<nbValues;j++) {
                    yLocalValues[i][j]-=yMin;
                }
                if (allRange) {
                    yLocalMax[i] -= (_xValues.length * yMin);
                }
                else {
                    yLocalMax[i] -= (nbValues * yMin);
                }
            }
        }

        
        StringBuffer string = new StringBuffer("Time");
        for (int j=0;j<names.length;j++) {
            string.append(","+names[j]);
        }
        w.write (string+newLine);

        
        for (int i=0;i<nbValues;i++) {
            string = new StringBuffer(xFormat.format(xLocalValues[i]));
            for (int j=0;j<names.length;j++) {
                double y = yLocalValues[j][i]/yLocalMax[j]*100.;
                if (y<0.) y=0.;
                if (y>100.) y=100.;
                string.append(","+getYValue(y));
            }
            w.write (string+newLine);
        }
        
        w.close();

         
    }
    String getYValue(double value) {
        String sValue;
        if (value == 0.) {
            sValue = "0.";
        }
        else if (Math.abs(value)>=0.1) {
            sValue = yFormat.format(value);
        }
        else {
            sValue = yScFormat.format(value);
        }
        return sValue;
    }

}
