/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * 
 * Copyright 2015 Riccardo Scorretti, Olivier Fabregues, Sandrine Demaneche, Elisabeth Navarro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */
package risaaligner.graphic;

import ampere.graphic2d.GMouseListener;
import ampere.graphic2d.GMouseMotionListener;
import ampere.graphic2d.GObject;
import ampere.graphic2d.GStyle;
import ampere.graphic2d.geom.GCurve;
import ampere.graphic2d.geom.GLine;
import ampere.util.primitives.Point2d;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import org.apache.commons.math3.analysis.interpolation.SplineInterpolator;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;
import risaaligner.RisaAligner;
import risaaligner.datas.DataSet;
import risaaligner.datas.Sample;
import risaaligner.datas.Spot;

/**
 *
 * @author olivier
 */
public class GuiLadder extends GuiDataSet implements GMouseListener,GMouseMotionListener,Printable {
    public Sample sample = null;
    
    //public TabPanel tabPanel;

    public double xMin = 0.;
    public double xMax = 0.;
    public double yMin = 0.;
    public double yMax = 0.;

    //GuiDataSet ladderPanel = null;
    
    GCurve gLadderCurve = null;
    GCurve gLadderRefCurve = null;
    GSpot[] gLadderSpots = null;
    GSpot[] gLadderRefSpots = null;
    GStyle spotStyle = null;
    GStyle spotRefStyle = null;

    BasicStroke lineStroke;
    BasicStroke levelStroke;
    GStyle ladderRefStyle;
    
    //double level = 0.;
    GLine gLevel;
    
    int viewMode=0;

    public GuiLadder(Sample sample) {
        this.sample = sample;
        
        setMaximumSize(new Dimension(10000,1000));
        setPreferredSize(new Dimension(1600,300));
         
        /*
         * creation des styles
         */
        lineStroke = new BasicStroke(2.f);
        float[] levelDash = {4,7};
        levelStroke = new BasicStroke(3.f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1.f,levelDash,0.f);

        ladderRefStyle = new GStyle(Color.GREEN,Color.RED,Color.red,lineStroke);
        spotStyle = new GStyle(null,null,Color.BLACK,Color.RED,Color.BLACK,Color.BLUE);
        spotRefStyle = new GStyle(null,null,Color.BLACK,Color.RED,Color.BLACK,Color.BLUE);

        /*
         * construction du ladderPanel
         */
        createLadderPanel();
        addGMouseListener(this);
        addGMouseMotionListener(this);
    }
    
    public void printLadder() {
         PrinterJob job = PrinterJob.getPrinterJob();
         job.setPrintable(this);
         boolean ok = job.printDialog();
         if (ok) {
             try {
                  job.print();
             } catch (PrinterException ex) {
              /* The job did not successfully complete */
             }
         }
        
    }
   
    //
            
    public void updateMinMax() {
        xMin = sample.ladderDataSet.xMin;
        xMax = sample.ladderDataSet.xMax;
        yMin = sample.ladderDataSet.yMin;
        yMax = sample.ladderDataSet.yMax;
        if (sample.hasReference()) {
            if (sample.refSample.ladderDataSet.xMin<xMin) xMin = sample.refSample.ladderDataSet.xMin;
            if (sample.refSample.ladderDataSet.xMax>xMax) xMax = sample.refSample.ladderDataSet.xMax;
            if (sample.refSample.ladderDataSet.yMin<yMin) yMin = sample.refSample.ladderDataSet.yMin;
            if (sample.refSample.ladderDataSet.yMax>yMax) yMax = sample.refSample.ladderDataSet.yMax;
        }
        setMinMax(xMin, xMax, yMin, yMax);
        addGrid(xMin, xMax, yMin, yMax);
    }


    private void createLadderPanel() {
        
        /*
         * construction du ladder superieur
         * avec grille et graduations et légendes
         */
        setLegendDown("time");
        setLegendLeft("Ladder");
        enableMouseWheel(true);
        enableMouseMoveScene(true);
        //ladderPanel.enableMouse(true);
        enableMouseMotion(true);
        enableHorizontalZoom(true,5,5,5);
        enableVerticalZoom(true,5,5,5);
       
        /*
         * construction de la grille et des données eventuelles
         */
        
        if (sample == null ) {
            setMinMax(0., 100., 0., 1.);
        }
        else {
            createLadder();
            /*
             * construction eventuelle du ladder de reference
             */
            if (sample.hasReference()) {
                addLadderRef ();
            }

            updateMinMax();
            
            /*
             * construction d'un seuil
             */
            GStyle levelStyle = new GStyle(Color.red,Color.green,Color.BLUE,levelStroke);
            gLevel = new GLine(getXMin(),sample.level,getXMax(),sample.level);
            gLevel.setStyle(levelStyle);
            gLevel.setSelectable(true);
            add(gLevel);
        
            /*
             * mise a jour de la couleur des spots suivant la valeur du seuil
             */
            updateGSpots();
        }
    }
    
    
    public void createLadder () {
        /*
         * construction de la courbe du ladder
         */
        DataSet dataSet = sample.ladderDataSet;
        gLadderCurve = new GCurve (dataSet.xValues,dataSet.yValues,true,false,true,false,false);
        GStyle ladderStyle = new GStyle(Color.BLUE,Color.RED,Color.red,lineStroke);
        gLadderCurve.setStyle(ladderStyle);
        add(gLadderCurve);

        /*
         * construction des sommets du ladder
         */
        gLadderSpots = new GSpot[sample.spots.length];
        for (int i=0;i<gLadderSpots.length;i++) {
            Spot spot = sample.spots[i];
            gLadderSpots[i] = new GSpot(spot,spotStyle,true,true,true,false,false);
        }
        addSpots(gLadderSpots);
    }

    public void addLadderRef () {
        if (gLadderRefCurve != null) {
            removeCurve(gLadderRefCurve);
            removeSpots(gLadderRefSpots);
        }
        /*
         * construction de la courbe du ladder de reference
         */
        if (sample.hasReference()) {
            DataSet dataSet = sample.refSample.ladderDataSet;
            gLadderRefCurve = new GCurve (dataSet.xValues,dataSet.yValues,true,false,true,false,false);
            gLadderRefCurve.setStyle(ladderRefStyle);
            add(gLadderRefCurve);

            /*
             * construction des sommets du ladder de reference
             */
            gLadderRefSpots = new GSpot[sample.refSpots.length];
            for (int i=0;i<gLadderRefSpots.length;i++) {
                Spot spot = sample.refSpots[i];
                gLadderRefSpots[i] = new GSpot(spot,spotRefStyle,true,true,true,false,false);
            }
            addSpots(gLadderRefSpots);
        }

        updateGSpots();
    }
    public void deleteLadderRef() {
        if (gLadderRefCurve != null) {
            removeCurve(gLadderRefCurve);
            removeSpots(gLadderRefSpots);
        }
    }
    
    
    public void updateLevel () {
        gLevel.setGCoords (getXMin(),sample.level,getXMax(),sample.level);
        gLevel.computeScale();
        updateGSpots();
        fireAllObjects();
        repaintCanvas();
    }

    public void updateSpotState(int spotType, int spotIndex) {
        GSpot gSpot = null;
        if (spotType == Spot.LADDER) {
            gSpot = gLadderSpots[spotIndex];
        }
        else if (spotType == Spot.LADDER_REF) {
            gSpot = gLadderRefSpots[spotIndex];
        }
        updateGSpot(gSpot);
        fireAllObjects();
        repaintCanvas();
    }

    private void updateGSpots() {
        //sample.removeAlignLadder();
        for (int i=0;i<gLadderSpots.length;i++) {
            GSpot gSpot = gLadderSpots[i];
            _updateGSpot(gSpot);
        }
        if (sample.hasReference()) {
            for (int i=0;i<gLadderRefSpots.length;i++) {
                GSpot gSpot = gLadderRefSpots[i];
                _updateGSpot(gSpot);
            }
        }
    }
    private void updateGSpot(GSpot gSpot) {
        //sample.removeAlignLadder();
        _updateGSpot(gSpot);
    }

    private void _updateGSpot(GSpot gSpot) {

        Spot spot = gSpot.spot;
        if (spot.state==Spot.ON) {
            gSpot.setNormalLineColor(Color.BLACK);
            gSpot.setNormalFillColor(Color.GREEN);
            gSpot.setRolloverFillColor(Color.GREEN);
            gSpot.setRolloverLineColor(Color.RED);
        }
        else if (spot.state==Spot.OFF) {
            gSpot.setNormalLineColor(Color.BLACK);
            gSpot.setNormalFillColor(Color.RED);
            gSpot.setRolloverFillColor(Color.RED);
            gSpot.setRolloverLineColor(Color.GREEN);
        }
        else {
            gSpot.setNormalLineColor(null);
            gSpot.setRolloverLineColor(Color.BLACK);
            if (spot.y < sample.level) {
                gSpot.setNormalFillColor(null);
                gSpot.setRolloverFillColor(null);
            }
            else {
                gSpot.setNormalFillColor(Color.BLUE);
                gSpot.setRolloverFillColor(Color.BLUE);
            }
        }
    }
    
    
    public int getNumberOfEnabledGSpots(Spot[] spots) {
        int nbSpots = 0;
        for (int i=0;i<spots.length;i++) {
            Spot spot = spots[i];
            if (spot.state==Spot.OFF) {
            }
            else if (spot.state==Spot.ON) {
                nbSpots++;
            }
            else if (spot.y > sample.level) {
                nbSpots++;
            }
        }
        return nbSpots;
    }
    public Spot[] getEnabledGSpots(Spot[] spots) {
        int nbSpots = getNumberOfEnabledGSpots(spots);
        Spot[] newSpots = new Spot[nbSpots];
        nbSpots=0;
        for (int i=0;i<spots.length;i++) {
            Spot spot = spots[i];
            if (spot.state==Spot.OFF) {
            }
            else if (spot.state==Spot.ON) {
                newSpots[nbSpots++] = spot;
            }
            else if (spot.y > sample.level) {
                newSpots[nbSpots++] = spot;
            }
        }
        return newSpots;
    }

    public int getAlignMode() {
        return viewMode;
    }

    public void viewAlignedDatas(int viewMode) {
        this.viewMode = viewMode;
        //if (!sample.isReference()) {
            if ((viewMode == RisaAligner.VIEW_LADDER_ALIGNED ||viewMode == RisaAligner.VIEW_DATAS_ALIGNED) && sample.isLadderAligned()) {
                this.viewMode = RisaAligner.VIEW_LADDER_ALIGNED;
                gLadderCurve.setGCoords(sample.ladderDataSet.xValues,sample.ladderDataSet.yAlignValues);
                fireObject(gLadderCurve);
                for (int i=0;i<gLadderSpots.length;i++) {
                    GSpot gSpot = gLadderSpots[i];
                    Spot spot = gSpot.spot;
                    gSpot.move(spot.alignX,spot.y);
                    fireObject(gSpot);
                }
            }
            else {
                this.viewMode = RisaAligner.VIEW_SAMPLE;
                gLadderCurve.setGCoords(sample.ladderDataSet.xValues,sample.ladderDataSet.yValues);
                fireObject(gLadderCurve);
                for (int i=0;i<gLadderSpots.length;i++) {
                    GSpot gSpot = gLadderSpots[i];
                    Spot spot = gSpot.spot;
                    gSpot.move(spot.x,spot.y);
                    fireObject(gSpot);
                }
            }
            
        //}
    }
    /*
    public int getViewMode() {
        return viewAlignMode;
    }
     * 
     */

    public void setRefSample(Sample refSample) {
        addLadderRef ();
        sample.setAlignable(isAlignable());
    }
    public void removeRefSample() {
        deleteLadderRef();
        sample.setAlignable(isAlignable());
    }
    public boolean isAlignable() {
        if (!sample.hasReference()) {
            return false;
        }
        else {
            /*
            if (sample.isReference()) {
                return false;
            }
             * 
             */
            int nbOfEnabledSpots = getNumberOfEnabledGSpots(sample.spots);
            int nbOfEnabledRefSpots = getNumberOfEnabledGSpots(sample.refSpots);
            if (nbOfEnabledSpots != nbOfEnabledRefSpots) return false;
            
            return true;
        }
    }
   
    public PolynomialSplineFunction getAlignFunction () {
        PolynomialSplineFunction psf = null;
        if (!sample.isReference()) {
            Spot[] spots = getEnabledGSpots(sample.spots);
            Spot[] refSpots = getEnabledGSpots(sample.refSpots);

            if (spots.length == refSpots.length) {

                double[] x = new double[spots.length+2];
                x[0] = sample.ladderDataSet.xMin;
                for (int i=0;i<spots.length;i++) x[i+1] = spots[i].x;
                x[spots.length+1] = sample.ladderDataSet.xMax;

                double[] y = new double[refSpots.length+2];
                y[0] = sample.refSample.ladderDataSet.xMin;
                for (int i=0;i<refSpots.length;i++) y[i+1] = refSpots[i].x;
                y[refSpots.length+1] = sample.refSample.ladderDataSet.xMax;

                SplineInterpolator interpolator = new SplineInterpolator();
                psf = interpolator.interpolate(x, y);

            }
        }
        return psf;
    }
    
    @Override
    public void mousePressed(MouseEvent e, Point2d point, GObject gLastObject) {
        if (RisaAligner.risaAligner.locked) {
            RisaAligner.risaAligner.writeError("Warning : Ladder alignment is locked"+'\n');
        }
    }
    
    @Override
    public void mouseClicked(MouseEvent e, Point2d point, GObject gLastObject) {
        if (!RisaAligner.risaAligner.locked) {
            if (gLastObject instanceof GSpot) {
                GSpot gSpot = (GSpot)gLastObject;
                Spot spot = gSpot.spot;
                RisaAligner.risaAligner.changeSpotState(spot.type,spot.index);
            }
        }
    }

    @Override
    public void mouseReleased(MouseEvent e, Point2d point, GObject gSelectedObject) {
        if (!RisaAligner.risaAligner.locked) {
            if (gSelectedObject!= null ) {
                if ( gSelectedObject == gLevel) {
                    double level = point.y;

                    if (level >= getYMax()) level = getYMax();
                    if (level <= getYMin()) level = getYMin();
                    //System.out.println("GuiDataSet:mouseDragged: level = "+level);
                    RisaAligner.risaAligner.setFinalLevel(level);
                }
            }
        }
    }

    @Override
    public void mouseEntered(MouseEvent e, Point2d point, GObject gLastObject) {
    }

    @Override
    public void mouseExited(MouseEvent e, Point2d point, GObject gLastObject) {
    }

    @Override
    public void mouseMoved(MouseEvent e, Point2d point, GObject gObject) {
    }

    @Override
    public void mouseDragged(MouseEvent e,int x0,int y0, Point2d point, GObject gSelectedObject, GObject gCurrentObject) {
        if (!RisaAligner.risaAligner.locked) {
            if (gSelectedObject!= null ) {
                if ( gSelectedObject == gLevel) {
                    double level = point.y;

                    if (level >= getYMax()) level = getYMax();
                    if (level <= getYMin()) level = getYMin();
                    //System.out.println("GuiDataSet:mouseDragged: level = "+level);
                    RisaAligner.risaAligner.setLevel(level);
                }
            }
        }
    }
}
