/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * 
 * Copyright 2015 Riccardo Scorretti, Olivier Fabregues, Sandrine Demaneche, Elisabeth Navarro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */
package risaaligner.graphic;

import ampere.graphic2d.GMouseListener;
import ampere.graphic2d.GMouseMotionListener;
import ampere.graphic2d.GObject;
import ampere.graphic2d.GScene;
import ampere.graphic2d.GStyle;
import ampere.graphic2d.geom.GCurve;
import ampere.graphic2d.geom.GImage;
import ampere.util.primitives.Point2d;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import risaaligner.RisaAligner;
import risaaligner.datas.DataSet;
import risaaligner.datas.Sample;

/**
 *
 * @author olivier
 */
public class GuiSample extends GuiDataSet  implements GMouseListener,GMouseMotionListener,Printable {
    
    
    public Sample sample = null;
    int viewMode;
    
    double[] xValues = null;
    double[] yValues = null;

    GCurve[] gCurves = null;
    BasicStroke lineStroke;

    DataSet currentDataSet;
    String dataSetName = null;
    

    GImage gLegend;
    int printMode =0;
    
    int first = 0;


    /** Creates new form GuiLadder 
    public GuiLadder(String repName,DataSet ladderRefDataSet,DataSet ladderDataSet,DataSet[] dataSets) {
        this.repName = repName;
        this.ladderRefDataSet = ladderRefDataSet;
        this.ladderDataSet = ladderDataSet;
        this.dataSets = dataSets;
     */
    public GuiSample() {
        
        
        setMaximumSize(new Dimension(10000,1000));
        setPreferredSize(new Dimension(1600,300));

        
        /*
         * creation des styles
         */
        lineStroke = new BasicStroke(2.f);
        float[] levelDash = {4,7};

        /*
         * construction du sample Panel
         */
        createSamplePanel();
        addGMouseListener(this);
        addGMouseMotionListener(this);

    }
    public void printSample() {
         PrinterJob job = PrinterJob.getPrinterJob();
         job.setPrintable(this);
         boolean ok = job.printDialog();
         if (ok) {
             try {
                  job.print();
             } catch (PrinterException ex) {
              /* The job did not successfully complete */
             }
         }
        
    }
    
    void createSamplePanel() {
        
        /*
         * construction du panel inferieur
         * avec grille et graduations et légendes
         */
        
        setLegendDown("time");
        setLegendLeft("Datas");
        enableMouseMotion(true);
        enableHorizontalZoom(true,5,5,5);
        enableVerticalZoom(true,5,5,5);
        enableMouseWheel(true);
        enableMouseMoveScene(true);


        setMinMax(0., 100., 0., 1.);
        
    }
    
    
    public void setSample(Sample sample,int aligned) {
        this.sample = sample;
        this.viewMode = aligned;
        removeAllInClip();
        
        if (sample != null) {
            /*
             * construction de la grille et des données eventuelles
             */

            setMinMax(sample.xMinOfDatasets, sample.xMaxOfDatasets, sample.yMinOfDatasets, sample.yMaxOfDatasets);
            addGrid(sample.xMinOfDatasets, sample.xMaxOfDatasets, sample.yMinOfDatasets, sample.yMaxOfDatasets);

            /*
             * ajout des courbes
             */
            gCurves = new GCurve[sample.dataSets.length];
            for (int i=0;i<sample.dataSets.length;i++) {
                
                fillTabValues(i);

                DataSet dataSet = sample.dataSets[i];
                GStyle curveStyle = new GStyle(dataSet.color,lineStroke);
                gCurves[i] = new GCurve (xValues,yValues,true,false,true,false,false);
                gCurves[i].setStyle(curveStyle);

                add(gCurves[i]);
            }
            setLegendLeft (sample.repName);

            gLegend = new GImage(getImage(),0.,0.,GObject.RIGHT,GObject.UP,GObject.RIGHT,GObject.UP);
            gLegend.setSelectable(true);
            addAlways(gLegend);
        }
        
    }
    public void viewAlignedDatas(int viewMode) {
        this.viewMode = viewMode;
        for (int i=0;i<sample.dataSets.length;i++) {
            fillTabValues(i);
            gCurves[i].setGCoords(xValues,yValues);
            fireObject(gCurves[i]);
        }
    }
    void fillTabValues (int noDataSet) {
        currentDataSet = sample.ladderDataSet;
        dataSetName = new String("Ladder");
        if (noDataSet>=0) {
            currentDataSet = sample.dataSets[noDataSet];
            dataSetName = sample.dataSets[noDataSet].sampleName;
        }
        if (viewMode == RisaAligner.VIEW_DATAS_ALIGNED && sample.isDatasAligned()) {
            xValues = currentDataSet.xValues;
            yValues = currentDataSet.yAlign2Values;
        }
        else if (viewMode == RisaAligner.VIEW_LADDER_ALIGNED && sample.isLadderAligned()) {
            xValues = currentDataSet.xValues;
            yValues = currentDataSet.yAlignValues;
        }
        else {
            xValues = currentDataSet.xValues;
            yValues = currentDataSet.yValues;
        }
    }
    
    /*
     * creation de la legende
     */
    private BufferedImage getImage(){

        //System.out.println("GUISample:getImage :");
        
        /*
         * calcul des couleurs et des tailles max des String
         */
        int lineLength = 30;
        int space = 10;
        int textWidth = 0;
        int textHeight = 0;
        int nbSamples = sample.dataSets.length;
        for (int i=0;i<nbSamples;i++) {
            DataSet dataSet = sample.dataSets[i];
            String name = dataSet.sampleName;
            Rectangle2D textBounds = getFont().getStringBounds(name,GScene.frc);
            int width = (int)(textBounds.getWidth());
            int height = (int)(textBounds.getHeight());
            if (i==0 || width>textWidth) textWidth = width;
            if (i==0 || height>textHeight)textHeight = height;
        }

        /*
         * creation de l'image sur deux colonnes
         */
        int nbColumns = (nbSamples+1)/2;
        int imageHeight = space + nbColumns*(textHeight+space);
        int imageWidth = 2 * (space + lineLength + space + textWidth + space);
        BufferedImage bufferedImage = new BufferedImage(imageWidth,imageHeight,BufferedImage.TYPE_INT_RGB);

        /*
         * trace du fond et du cadre
         */
        Graphics g = bufferedImage.getGraphics();
        g.setColor(Color.WHITE);
        g.fillRect(0,0, imageWidth-1, imageHeight-1);
        g.setColor(Color.BLACK);
        g.drawRect(0,0, imageWidth-1, imageHeight-1);
        
        /*
         * trace de la legende sur deux colonnes
         */
        int x1 = space;
        int x2 = x1 + lineLength;
        int x3 = x2 + space;
        int x4 = x3 + textWidth + space;
        int x5 = x4 + lineLength;
        int x6 = x5 + space;
        int y = space+textHeight;
        for (int i=0;i<nbColumns;i++) {
            DataSet dataSet = sample.dataSets[i];
            String name = dataSet.sampleName;
            g.setColor(dataSet.color);
            g.drawLine(x1,y,x2,y);
            g.setColor(Color.BLACK);
            g.drawString(name,x3,y);
            
            int index = i+nbColumns;
            if (index <nbSamples) {
                dataSet = sample.dataSets[index];
                name = dataSet.sampleName;
                g.setColor(dataSet.color);
                g.drawLine(x4,y,x5,y);
                g.setColor(Color.BLACK);
                g.drawString(name,x6,y);
            }
            y+=(textHeight+space);
        }
        return bufferedImage;
   }
    

    @Override
    public void mouseClicked(MouseEvent e, Point2d point, GObject gLastObject) {
        //guiSample.mouseClicked(e, point, gLastObject);
    }

    @Override
    public void mousePressed(MouseEvent e, Point2d point, GObject gLastObject) {
    }

    @Override
    public void mouseReleased(MouseEvent e, Point2d point, GObject gLastObject) {
    }

    @Override
    public void mouseEntered(MouseEvent e, Point2d point, GObject gLastObject) {
    }

    @Override
    public void mouseExited(MouseEvent e, Point2d point, GObject gLastObject) {
    }

    @Override
    public void mouseMoved(MouseEvent e, Point2d point, GObject gObject) {
    }

    @Override
    public void mouseDragged(MouseEvent e,int x0,int y0, Point2d point, GObject gSelectedObject, GObject gCurrentObject) {
        if (gSelectedObject!= null ) {
            if ( gSelectedObject == gLegend) {
                double dX = (double)(e.getX() - x0);
                double dY = (double)(e.getY() - y0);
                gLegend.moveCoords(dX,dY);
                gLegend.computeScale();
                fireAllObjects();
                repaintCanvas();
            }
        }
    }
    
}
