/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 * 
 * Copyright 2015 Riccardo Scorretti, Olivier Fabregues, Sandrine Demaneche, Elisabeth Navarro.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */
package risaaligner.graphic;

import ampere.graphic2d.Curve.GSetOfShades;
import ampere.graphic2d.GMouseListener;
import ampere.graphic2d.GMouseMotionListener;
import ampere.graphic2d.GObject;
import ampere.graphic2d.GObjectFinder;
import ampere.graphic2d.GScene;
import ampere.graphic2d.GStyle;
import ampere.graphic2d.geom.GShadeUser;
import ampere.util.primitives.Point2d;
import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import risaaligner.RisaAligner;
import risaaligner.datas.DataSet;
import risaaligner.datas.Sample;
import risaaligner.datas.SpotLine;

/**
 *
 * @author olivier
 */
public class ShadeSample extends GSetOfShades implements GMouseListener,GMouseMotionListener,GObjectFinder,Printable  {

    public Sample sample;
    int viewMode;
            
    int nbShades = 0;
    GShadeUser[] shades;
    double xMin = 0.;
    double xMax = 1.;
    double yMin = 0.;
    double yMax = 1.;
    
    Color shadeColor = Color.BLACK;

    String[] shadeNames;
    double[] shadeNamesPositions;
    double[] x1Values = null;
    double[] x2Values = null;
    
    /*
     * variables pour le dataset en cours de traitement
     */
    DataSet currentDataSet;
    String shadeName = null;
    double[] xValues = null;
    double[] yValues = null;
    SpotLine[] spotLines = null;
    
    double xMinCurrent = 0.;
    double xMaxCurrent = 0.;
    double yMinCurrent = 0.;
    double yMaxCurrent = 0.;
    double xMinTotal = 0.;
    double xMaxTotal = 1.;
    double yMinTotal = 0.;
    double yMaxTotal = 1.;
    /*
    int[] spotIndexesCurrent = null;
    int[] spotStatesCurrent = null;
     * 
     */
    boolean spotLineVisible = false;
    
    int marginShade = 5;

    private int xDrag = 0;
    private int yDrag = 0;

    int selectedShade=-1;
    DataSet selectedDataSet;
    int selectedSpot;
    double level = 10.;

    boolean draggingSpot = false;
     /*   
    double yMinLocal = 0.;
    double yMaxLocal = 0.;
    String nameLocal = "";
     * 
     */

    public ShadeSample() {

        /*
         * configuration de la scene
         */
        /*
         * creation et choix des formats
         */
        NumberFormat formatY = NumberFormat.getInstance();
        if (formatY instanceof DecimalFormat) {
            ((DecimalFormat)formatY).applyPattern("000");
        }
        setFormatYUnits(formatY);

        setHUnitsAlignement (GObject.CENTER,GObject.UP);
        Font labelFont = new Font ("Dialog", Font.BOLD, 14);
        GStyle labelStyle = new GStyle(Color.BLACK,Color.WHITE,Color.BLACK,Color.RED,Color.BLACK,Color.BLUE,labelFont,Color.BLACK);
        Font unitsFont = new Font ("Dialog", Font.BOLD, 10);
        GStyle unitsStyle = new GStyle(Color.BLACK,Color.WHITE,Color.BLACK,Color.RED,Color.BLACK,Color.BLUE,unitsFont,Color.BLACK);
        
        setLabelStyle (labelStyle);
        setUnitsStyle (unitsStyle);

        GStyle cadreStyle = new GStyle(Color.black,Color.BLACK,Color.BLACK,null);
        setLineBorder(cadreStyle);

        setDrawingMode(GScene.SIMPLE_BUFFER);

        setMargins(5,15,15,5);
        enableVerticalZoom(true,5,5,5);
        enableWheelVerticalZoom(true);
        //enableMouseMoveScene(true);
        
        setLegendDown ("samples");
        addVerticalUnits();
        setXMargeLabel(5);
        setYMargeLabel(5);

        /*
         * config de l interactivite
         */
        addGMouseListener(this);
        addGMouseMotionListener(this);
        enableMouseMotion(true);
        setObjectFinder(this);
        
        
        xMin = 0.;
        xMax = 1.;
        //yMin = sample.xMinTotal;
        //yMax = sample.xMaxTotal;

    }

    public void setSample(Sample sample,int viewMode) {
        this.sample = sample;
        level = (sample.xMaxTotal-xMinTotal)/100.;
        this.viewMode = viewMode;
        shades = null;
        nbShades = 0;
        removeAllShades();
  
        if (sample != null) {

            //yMin = sample.xMinTotal;
            //yMax = sample.xMaxTotal;

            nbShades = 1+sample.dataSets.length;

            xMin = 0.;
            xMax = 1.;
            double shadeWidth = (xMax-xMin)/ nbShades;
            shades = new GShadeUser[nbShades];
            shadeNamesPositions = new double[nbShades]; 
            shadeNames = new String[nbShades]; 
            x1Values = new double[nbShades];
            x2Values = new double[nbShades];
            
            /*
             * create shade image
             */

            for (int i=0;i<nbShades;i++) {

                fillTabValues(i);

                x1Values[i] = xMin + i*shadeWidth;
                x2Values[i] = xMin + (i+1)*shadeWidth;
                shades[i] = new GShadeUser (x1Values[i],x2Values[i],xValues,shadeColor,yValues);
                shades[i].setSelectable(true);
                shades[i].setMargins(marginShade,marginShade);
                shades[i].setBackground(Color.WHITE);
                shades[i].setMargins(5,5);
                shades[i].setBackground(Color.WHITE);
                shadeNames[i] = shadeName;
                shadeNamesPositions[i] = (x1Values[i]+x2Values[i])/2.;
                shades[i].createSpotLines(spotLines);
            }

            setShades (shades);
            yMin = xMinTotal;
            yMax = xMaxTotal;
            setMinMax(xMin,xMax,yMin,yMax);
            setLegendLeft (sample.repName);

            /*
             * config de l echelle verticale
             */
            addHorizontalUnits(shadeNamesPositions,shadeNames);
        }
    }
    public void viewAlignedDatas (int viewMode) {
        this.viewMode = viewMode;
        for (int i=0;i<nbShades;i++) {
            fillTabValues(i);

            shades[i].setValues(xValues, shadeColor, yValues);
            if (spotLines != null) shades[i].setSpotValues(spotLines);
            fireObject(shades[i]);
        }
    }
    void fillTabValues (int noDataSet) {
        double [] xLocalValues = null;
        double [] yLocalValues = null;
        spotLines = null;
        
        currentDataSet = sample.ladderDataSet;
        shadeName = new String("Ladder");
        if (noDataSet>0) {
            currentDataSet = sample.dataSets[noDataSet-1];
            shadeName = sample.dataSets[noDataSet-1].sampleName;
        }

        if (viewMode == RisaAligner.VIEW_DATAS_ALIGNED && sample.isDatasAligned()) {
            xLocalValues = currentDataSet.xValues;
            yLocalValues = currentDataSet.yAlign2Values;
            xMinCurrent = currentDataSet.xMin;
            xMaxCurrent = currentDataSet.xMax;
            yMinCurrent = currentDataSet.yMin;
            yMaxCurrent = currentDataSet.yMax;
            spotLines = currentDataSet.spotLines;
            for (int i=0;i<spotLines.length;i++)spotLines[i].xIndex = spotLines[i].xIndex1;
            spotLineVisible = true;
        }
        else if (viewMode == RisaAligner.VIEW_LADDER_ALIGNED && sample.isLadderAligned()) {
            xLocalValues = currentDataSet.xValues;
            yLocalValues = currentDataSet.yAlignValues;
            xMinCurrent = currentDataSet.xMin;
            xMaxCurrent = currentDataSet.xMax;
            yMinCurrent = currentDataSet.yMin;
            yMaxCurrent = currentDataSet.yMax;
            spotLines = currentDataSet.spotLines;
            for (int i=0;i<spotLines.length;i++)spotLines[i].xIndex = spotLines[i].xIndex0;
            spotLineVisible = true;
        }
        else {
            xLocalValues = currentDataSet.xValues;
            yLocalValues = currentDataSet.yValues;
            xMinCurrent = currentDataSet.xMin;
            xMaxCurrent = currentDataSet.xMax;
            yMinCurrent = currentDataSet.yMin;
            yMaxCurrent = currentDataSet.yMax;
            spotLines = currentDataSet.spotLines;
            for (int i=0;i<spotLines.length;i++)spotLines[i].xIndex = 0;
            spotLineVisible = false;
        }
        
        if (yMinCurrent <0) yMinCurrent = 0.;
        xValues = new double[xLocalValues.length];
        yValues = new double[yLocalValues.length];
        for (int j=0;j<xLocalValues.length;j++) {
            xValues[j] = (xLocalValues[j]);
            yValues[j] = (yLocalValues[j]-yMinCurrent)/(yMaxCurrent-yMinCurrent);
            if (yValues[j] <0. ) yValues[j] = 0.;
            if (yValues[j] >1. ) yValues[j] = 1.;
        }
        
        if (noDataSet==0 || xMinCurrent<xMinTotal) xMinTotal = xMinCurrent;
        if (noDataSet==0 || xMaxCurrent<xMaxTotal) xMaxTotal = xMaxCurrent;

    }

    public void removeSample() {
        this.sample = null;
        removeAllShades();
    }
    public int getAlignedMode() {
        return viewMode;
    }
    public boolean isAligned2() {
        return sample.isDatasAligned();
    }
    
    public void createSpotLines() {
        
        /*
         * create shade image
         */

        for (int i=0;i<nbShades;i++) {
            fillTabValues(i);
            shades[i].createSpotLines(spotLines);
        }
        fireAllObjects();
        scaleAndRepaint();
    }
    public void updateSpotLines() {
        
        /*
         * create shade image
         */

        for (int i=0;i<nbShades;i++) {
            fillTabValues(i);
            shades[i].updateSpotLines();
        }
        fireAllObjects();
        scaleAndRepaint();
    }
    
    public void printSample() {
         PrinterJob job = PrinterJob.getPrinterJob();
         job.setPrintable(this);
         boolean ok = job.printDialog();
         if (ok) {
             try {
                  job.print();
             } catch (PrinterException ex) {
              /* The job did not successfully complete */
             }
         }
        
    }
    @Override
    public void mousePressed(MouseEvent e, Point2d point2d, GObject gObject) {
        Point point = e.getPoint();
        xDrag = point.x;
        yDrag = point.y;
        selectedShade = -1;
        selectedSpot = -1;
        
        if (viewMode == RisaAligner.VIEW_LADDER_ALIGNED || viewMode == RisaAligner.VIEW_DATAS_ALIGNED ) {
            if (gObject != null) {
                if (gObject instanceof GShadeUser) {
                    GShadeUser gShadeUser = (GShadeUser) gObject;
                    for (int i=0;i<shades.length;i++) {
                        if (shades[i]==gShadeUser) {
                            selectedShade = i;
                            break;
                        }
                    }
                    if (selectedShade >=0) {
                        selectedSpot = gShadeUser.getLineSpotForValue(point, point2d, level);
                    }
                }
            }
        }
        else {
            RisaAligner.risaAligner.writeError("Warning : ladder must be aligned before setting levels"+'\n');
        }
    }

    @Override
    public void mouseClicked(MouseEvent e, Point2d point2d, GObject gObject) {
        
        // si c est le bouton de droite
        if ((e.getModifiers() & MouseEvent.BUTTON3_MASK) == MouseEvent.BUTTON3_MASK) {
            initZoom();
        }

        // si c est le bouton de gauche 
        else {
            // si c est le degrade du ladder
            if (selectedShade == 0) {
                if (selectedSpot >=0) {
                    RisaAligner.risaAligner.removeSpotLine(selectedSpot);
                }
                else {
                    RisaAligner.risaAligner.createSpotLineAuto(point2d.y);
                }
            }
            // si c est un autre
            else if (selectedShade > 0) {
                if (selectedSpot >=0) {
                    RisaAligner.risaAligner.changeSpotLineState(selectedShade-1,selectedSpot);
                }
            }
        }
    }
    


    @Override
    public void mouseReleased(MouseEvent e, Point2d point2d, GObject gLastObject) {
            // si un spot est selectionné
        if (draggingSpot ) {
            RisaAligner.risaAligner.moveFinalSpotLine(selectedShade-1,selectedSpot,point2d.y);
        }
        draggingSpot = false;
    }

    @Override
    public void mouseEntered(MouseEvent e, Point2d point, GObject gLastObject) {
    }

    @Override
    public void mouseExited(MouseEvent e, Point2d point, GObject gLastObject) {
    }


    @Override
    public void mouseMoved(MouseEvent e, Point2d point, GObject gObject) {
    }

    @Override
    public void mouseDragged(MouseEvent e, int x0, int y0, Point2d point2d, GObject selectedGObject, GObject gCurentObject) {
        boolean localToDraw = false;
        int x = e.getPoint().x;
        int dx = x-xDrag;
        int y = e.getPoint().y;
        int dy = y-yDrag;

        if (dx == 0 && dy == 0) return;
        
        // si c est le bouton de gauche
        if ((e.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK) {
            
            // si un spot est selectionné
            if (selectedSpot >=0) {
                draggingSpot = true;
                RisaAligner.risaAligner.moveSpotLine(selectedShade-1,selectedSpot,point2d.y);
            }
            
            // pas d'objets a deplacer, on deplace la scene
            else {
                moveZoom (dx,dy);
                localToDraw = true;
            }
            
        }
        if (localToDraw)  {
            scaleAndRepaint();
            //System.out.println ("ShadeSample : mouseDragged "+dx+" ,dy = "+dy);
            xDrag = x;
            yDrag = y;
        }
    }
    @Override
    public GObject findObject(Point point,Point2d point2d) {
        GObject object=null;
        for (int i=0;i<nbShades;i++) {
            if (shades[i].contains(point)){
                object = shades[i];
                break;
            }
        }
        return object;
    }
}
